// -------------------------------------------------------*- C++ -*----------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloarrayexpriH
#define __CONCERT_iloarrayexpriH

#if !defined(__CONCERT_iloforallbaseH)
# include <ilconcert/iloforallbase.h>
#endif
#if !defined(__CONCERT_ilomapiH)
# include <ilconcert/ilomapi.h>
#endif



/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntSetByExtensionExprI </p>
**/
class IloIntSetByExtensionExprI : public IloIntCollectionExprArgI {
	ILOEXTRDECL
		IloMapItemArray _comp;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	// no need to lock e, this is done in the comprehension.
	IloIntSetByExtensionExprI(IloEnvI* env, IloMapItemArray e);
	virtual ~IloIntSetByExtensionExprI();

	IloMapItemArray getArray() const { return _comp; }

	/** <p> . </p> */
	void display(ILOSTD(ostream)& out) const;
	/** <p> . </p> */
	virtual IloNum eval(const IloAlgorithm alg) const;
	/** <p> . </p> */
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	virtual IloBool isInteger() const;
};
//



/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloSymbolSetByExtensionExprI </p>
**/
class IloSymbolSetByExtensionExprI : public IloSymbolCollectionExprArgI {
	ILOEXTRDECL
		IloMapItemArray _comp;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	// no need to lock e, this is done in the comprehension.
	IloSymbolSetByExtensionExprI(IloEnvI* env, IloMapItemArray e);

	virtual ~IloSymbolSetByExtensionExprI();

	IloMapItemArray getArray() const { return _comp; }
	virtual IloEncoderI* getEncoder() const{ return getEnv()->getSymbolEncoder(); }
	ILOEXTROTHERDECL
};
//



/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloSymbolSetByExtensionExprI </p>
**/
class IloTupleSetByExtensionExprI : public IloTupleSetExprArgI {
	ILOEXTRDECL
	IloTupleSchemaI* _schema;
	IloMapItemArray _comp;

	//IloBool _hasReference;
	//IloIntArray _idx;
	//IloArray<IloDiscreteDataCollectionI*> _references;

	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	// no need to lock e, this is done in the comprehension.
	IloTupleSetByExtensionExprI(IloEnvI* env, IloTupleSchemaI* schema, IloMapItemArray e);
	//IloBool hasReference() const { return _hasReference;}
	//IloIntArray getReferenceIndexes() const{ return _idx;}
	//IloArray<IloDiscreteDataCollectionI*> getReferences() const{ return _references;}
	//void setReferences(IloIntArray idx, IloArray<IloDiscreteDataCollectionI*> references);
	virtual ~IloTupleSetByExtensionExprI();
	IloTupleSchemaI* getSchema() const { return _schema; }
	IloMapItemArray getArray() const { return _comp; }
	virtual IloEncoderI* getEncoder() const{ return getEnv()->getSymbolEncoder(); }
	ILOEXTROTHERDECL
};




/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumSetByExtensionExprI </p>
**/
class IloNumSetByExtensionExprI : public IloNumCollectionExprArgI {
	ILOEXTRDECL
		IloMapItemArray _comp;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	// no need to lock e, this is done in the comprehension.
	IloNumSetByExtensionExprI(IloEnvI* env, IloMapItemArray e);
	virtual ~IloNumSetByExtensionExprI();

	IloMapItemArray getArray() const { return _comp; }

	/** <p> . </p> */
	void display(ILOSTD(ostream)& out) const;
	/** <p> . </p> */
	virtual IloNum eval(const IloAlgorithm alg) const;
	/** <p> . </p> */
	virtual IloExtractableI* makeClone(IloEnvI*) const;
};
//

/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntSetByExtensionExprIIterator </p>
**/
class IloIntSetByExtensionExprIIterator : public IloIntDataIterator {
	IloIntSetByExtensionExprI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> */
	IloIntSetByExtensionExprIIterator(IloGenAlloc* heap, 
		const IloIntSetByExtensionExprI* expr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> */
	virtual ~IloIntSetByExtensionExprIIterator();
	/** <p> . </p> */
	virtual IloBool next();
	/** <p> . </p> */
	virtual IloInt recomputeMin() const;
	/** <p> . </p> */
	virtual IloInt recomputeMax() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloInt intMin, IloInt intMax, IloBool catchInvalidCollection = IloFalse);
};



/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumSetByExtensionExprIIterator </p>
**/
class IloNumSetByExtensionExprIIterator : public IloNumDataIterator {
	IloNumSetByExtensionExprI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> **/
	IloNumSetByExtensionExprIIterator(IloGenAlloc* heap, 
		const IloNumSetByExtensionExprI* expr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> **/
	virtual ~IloNumSetByExtensionExprIIterator();
	/* <p> . </p> */
	virtual IloBool next();
	/* <p> . </p> */
	virtual IloNum recomputeLB() const;
	/* <p> . </p> */
	virtual IloNum recomputeUB() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloNum numLB, IloNum numUB, IloBool catchInvalidCollection = IloFalse);
};


/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloSymbolSetByExtensionExprIIterator </p>
**/
class IloSymbolSetByExtensionExprIIterator : public IloAnyDefaultDataIterator {
	IloSymbolSetByExtensionExprI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> */
	IloSymbolSetByExtensionExprIIterator(IloGenAlloc* heap, 
		const IloSymbolSetByExtensionExprI* expr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> */
	virtual ~IloSymbolSetByExtensionExprIIterator();
	/** <p> . </p> */
	virtual IloBool next();
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloAny value, IloBool catchInvalidCollection = IloFalse);
};


#endif
