// -------------------------------------------------------*- C++ -*----------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilointcollexpriH
#define __CONCERT_ilointcollexpriH

#if !defined(__CONCERT_iloforallbaseH)
# include <ilconcert/iloforallbase.h>
#endif



class IloTupleIndexI;

class IloIntCollectionExprGeneratorI;

/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntCollectionIndexI </p>
**/ 
class IloIntCollectionIndexI : public IloIntExprI {
  ILOEXTRDECL
  void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloIntCollectionIndexI(IloEnvI* env, const char* name=0); 
  virtual ~IloIntCollectionIndexI();
  IloInt getKey() const;
  virtual void errorNotSubstituted() const;
  /* IloIntGeneratorI* within(IloIntCollectionI* coll) const; */
  IloNum eval(const IloAlgorithm alg) const;
  virtual IloBool isInteger() const;
  ILOEXTROTHERDECL 
};

/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntCollectionTupleCellIndexI </p>
**/
class IloIntCollectionTupleCellExprI : public IloIntCollectionExprI {
	ILOEXTRDECL
private:
	IloTupleExprI* _tuple;
	IloInt _colIndex;
	IloSymbolI* _colName;
	void visitSubExtractables(IloExtractableVisitor* v);
public:
	/* <p> . </p> */
	IloIntCollectionTupleCellExprI(IloEnvI* env, IloTupleExprI* tuple, IloInt colIndex);
	/* <p> . </p> */
	IloIntCollectionTupleCellExprI(IloEnvI* env, IloTupleExprI* tuple, IloSymbolI* colName);
  /* <p> . </p> */
  virtual ~IloIntCollectionTupleCellExprI();
	/* <p> . </p> */
	IloTupleExprI* getTuple() const { return _tuple; }
	/* <p> . </p> */
	IloInt getColumnIndex() const { return _colIndex; }
	/** <p> . </p> */
	IloSymbolI* getColumnName() const { return _colName; }
	/** <p> . </p> */
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloBool isInteger() const;
	ILOEXTROTHERDECL
};

/** 
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntCollectionExprGeneratorI </p>
**/
class IloIntCollectionExprGeneratorI : public IloIntGeneratorI {
	ILOEXTRDECL
private:
	IloIntCollectionExprArgI* _coll;
	void visitSubExtractables(IloExtractableVisitor* v);
public:
	/** <p> . </p> */
	IloIntCollectionExprGeneratorI(IloEnvI* env, IloIntIndexI* x, IloIntCollectionExprArgI* expr);
	/** <p> . </p> */
	IloIntCollectionExprArgI* getCollection() const { return _coll; }
	/** <p> . </p> */
	virtual IloBool generatesDuplicates() const;
	ILOEXTROTHERDECL
};

/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloEvalIntCollectionExprIIterator </p>
**/
class IloEvalIntCollectionExprI;
class IloEvalIntCollectionExprIIterator : public IloIntDefaultDataIterator {
	IloEvalIntCollectionExprI*  _expr;
public:
	/** <p> . </p> **/
	IloEvalIntCollectionExprIIterator(IloGenAlloc* heap, 
		IloEvalIntCollectionExprI* expr);
	/** <p> . </p> **/
	virtual ~IloEvalIntCollectionExprIIterator();
	/* <p> . </p> */
	//virtual IloBool next();
	virtual IloInt recomputeMin() const;
	/* <p> . </p> */
	virtual IloInt recomputeMax() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloInt min, IloInt max, IloBool catchInvalidCollection = IloFalse);
};	

/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntRangeExprI </p>
**/
class IloIntRangeExprI : public IloIntCollectionExprArgI {
	ILOEXTRDECL
	IloIntExprI* _lb;
	IloIntExprI* _ub;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	/** <p> . </p> */
	IloIntRangeExprI(IloEnvI* env, IloIntExprI* lb, IloIntExprI* ub) : 
	  IloIntCollectionExprArgI(env), _lb(lb->intLockExpr()), _ub(ub->intLockExpr()) {}
	  virtual ~IloIntRangeExprI();
	  /** <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /** <p> . </p> */
	  virtual IloNum eval(const IloAlgorithm alg) const;
	  /** <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;
	  /** <p> . </p> */
	  IloIntExprI* getLB() const { return _lb; }
	  /** <p> . </p> */
	  IloIntExprI* getUB() const { return _ub; }
	  virtual IloBool isInteger() const;
};




/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntRangeExprI </p>
**/
class IloIntRangeAsIntSetExprI : public IloIntRangeExprI {
	// temporary for frank.
	IloIntRangeExprI* _expr;
	ILOEXTRDECL
public:
	/** <p> . </p> */
	IloIntRangeAsIntSetExprI(IloEnvI* env, IloIntExprI* lb, IloIntExprI* ub) : 
	  IloIntRangeExprI(env, lb, ub), _expr(0){}
	// temporary for frank.
	IloIntRangeAsIntSetExprI(IloEnvI* env, IloIntRangeExprI* R) : 
	  IloIntRangeExprI(env, R->getLB(), R->getUB()), _expr(R){}
	  virtual ~IloIntRangeAsIntSetExprI(){}
	  /** <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /** <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;

  	  // temporary for frank.
	  IloIntRangeExprI* getRange() const{
		  return _expr;
	  }
	  IloIntRangeExprI* makeRange() const{
		  return new (getEnv()) IloIntRangeExprI(getEnv(), getLB(), getUB());
	  }
};


/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntRangeExprIIterator </p>
**/
class IloIntRangeExprIIterator : public IloIntDataIterator {
	IloIntRangeExprI*  _rangeexpr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> */
	IloIntRangeExprIIterator(IloGenAlloc* heap, 
		const IloIntRangeExprI* rangeexpr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> */
	virtual ~IloIntRangeExprIIterator();
	/** <p> . </p> */
	virtual IloBool next();
	/** <p> . </p> */
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	/** <p> . </p> */
	virtual void reset(IloInt min, IloInt max, IloBool catchInvalidCollection = IloFalse);
	/** <p> . </p> */
	virtual IloInt recomputeMin() const;
	/** <p> . </p> */
	virtual IloInt recomputeMax() const;
};	


/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntCollectionTupleCellExprIIterator </p>
**/
class IloIntCollectionTupleCellExprIIterator : public IloIntDefaultDataIterator {
	IloIntCollectionTupleCellExprI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> **/
	IloIntCollectionTupleCellExprIIterator(IloGenAlloc* heap, 
		const IloIntCollectionTupleCellExprI* rangeexpr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> **/
	virtual ~IloIntCollectionTupleCellExprIIterator();
	/* <p> . </p> */
	//virtual IloBool next();
	/* <p> . </p> */
	virtual IloInt recomputeMin() const;
	virtual IloInt recomputeMax() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloInt min, IloInt max, IloBool catchInvalidCollection = IloFalse);
};	


/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntCollectionIndexIIterator </p>
**/
class IloIntCollectionIndexIIterator : public IloIntDefaultDataIterator {
	IloIntCollectionIndexI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> **/
	IloIntCollectionIndexIIterator(IloGenAlloc* heap, 
		const IloIntCollectionIndexI* rangeexpr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> **/
	virtual ~IloIntCollectionIndexIIterator();
	/* <p> . </p> */
	//virtual IloBool next();
	/* <p> . </p> */
	virtual IloInt recomputeMin() const;
	/* <p> . </p> */
	virtual IloInt recomputeMax() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloInt min, IloInt max, IloBool catchInvalidCollection = IloFalse);
};	



class IloIntCollectionSubMapExprI;
/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloSymbolCollectionIndexIIterator </p>
**/
class IloIntCollectionSubMapExprIIterator : public IloIntDefaultDataIterator {
	const IloIntCollectionSubMapExprI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> **/
	IloIntCollectionSubMapExprIIterator(IloGenAlloc* heap, 
		const IloIntCollectionSubMapExprI* map, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> **/
	virtual ~IloIntCollectionSubMapExprIIterator();
	/* <p> . </p> */
	//virtual IloBool next();
	virtual IloInt recomputeMin() const;
	/* <p> . </p> */
	virtual IloInt recomputeMax() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloInt min, IloInt max, IloBool catchInvalidCollection = IloFalse);
};	



/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntAggregateSetExprI </p>
**/
class IloIntAggregateSetExprI : public IloIntCollectionExprArgI {
	ILOEXTRDECL
	IloExtendedComprehensionI* _comp;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	// no need to lock e, this is done in the comprehension.
	IloIntAggregateSetExprI(IloEnvI* env, IloIntExprI* e, IloGeneratorArray g);
	IloIntAggregateSetExprI(IloExtendedComprehensionI* comp);

	virtual ~IloIntAggregateSetExprI();
	
	  IloExtendedComprehensionI* getComprehension() const { return _comp; }

	  /** <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /** <p> . </p> */
	  virtual IloNum eval(const IloAlgorithm alg) const;
	  /** <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;
	  virtual IloBool isInteger() const;
	  IloIntExprI* getBody() const { return (IloIntExprI*)_comp->getExtent(); }
};
//




/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntAggregateUnionSetExprI </p>
**/
class IloIntAggregateUnionSetExprI : public IloIntCollectionExprArgI {
	ILOEXTRDECL
	IloExtendedComprehensionI* _comp;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	// no need to lock e, this is done in the comprehension.
	IloIntAggregateUnionSetExprI(IloEnvI* env, IloIntCollectionExprI* e, IloGeneratorArray g);
	IloIntAggregateUnionSetExprI(IloExtendedComprehensionI* comp);

	virtual ~IloIntAggregateUnionSetExprI();
	
	  IloExtendedComprehensionI* getComprehension() const { return _comp; }

	  /** <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /** <p> . </p> */
	  virtual IloNum eval(const IloAlgorithm alg) const;
	  /** <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;
	  virtual IloBool isInteger() const;
	  IloIntExprI* getBody() const { return (IloIntExprI*)_comp->getExtent(); }
};
//




/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntAggregateInterSetExprI </p>
**/
class IloIntAggregateInterSetExprI : public IloIntCollectionExprArgI {
	ILOEXTRDECL
	IloExtendedComprehensionI* _comp;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	// no need to lock e, this is done in the comprehension.
	IloIntAggregateInterSetExprI(IloEnvI* env, IloIntCollectionExprI* e, IloGeneratorArray g);
	IloIntAggregateInterSetExprI(IloExtendedComprehensionI* comp);

	virtual ~IloIntAggregateInterSetExprI();
	
	  IloExtendedComprehensionI* getComprehension() const { return _comp; }

	  /** <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /** <p> . </p> */
	  virtual IloNum eval(const IloAlgorithm alg) const;
	  /** <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;
	  virtual IloBool isInteger() const;
	  IloIntCollectionExprI* getBody() const { return (IloIntCollectionExprI*)_comp->getExtent(); }
};
//



class IloIntCollectionConstI : public IloIntCollectionExprI {
	ILOEXTRDECL
private:
	IloIntCollectionI* _coll;
  IloBool _ownsColl;
	void visitSubExtractables(IloExtractableVisitor* v);
public:
	/* <p> . </p> */
	IloIntCollectionConstI(IloEnvI* env, IloIntCollectionI* coll, IloBool ownsColl=IloFalse);
  /* <p> . </p> */
	virtual ~IloIntCollectionConstI();
	/* <p> . </p> */
	IloIntCollectionI* getCollection() const { return _coll; }
	/** <p> . </p> */
	virtual IloNum eval(const IloAlgorithm alg) const{
		throw IloWrongUsage("can not evaluate");
		ILOUNREACHABLE(return 0;)
	}
	virtual IloBool isInteger() const{ return IloTrue;}
	ILOEXTROTHERDECL
};


//

#endif
