// --------------------------------------------------*- C++ -*---------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloevaluatorH
#define __CONCERT_iloevaluatorH

#if !defined(__CONCERT_iloenvH)
#include <ilconcert/iloenv.h>
#endif
#if !defined(__CONCERT_iloforallH)
#include <ilconcert/iloforall.h>
#endif
#if !defined(__CONCERT_ilocollexprH)
#include <ilconcert/ilocollexpr.h>
#endif

class IloModelEvaluatorI;
class IloEvalExprI;
class IloEvalTupleExprI;
class IloEvalIteratorI;
class IloMapI;
class IloMapExtractIndexI;
class IloTupleCollectionI;
class IloTupleCollection;
class IloMapIndexArray;
class IloMapIndex;



#ifdef OPL_1748
class IloMismatchDExprSubstitution: public IloException {
private:
  IloExtractableI* _expr;
  IloMapIndexArray _indexes;
  IloMapIndexArray _values;
public:
	virtual void print(ILOSTD(ostream)& out) const{
		out << _message;
		if (_expr) _expr->display(out);
	}
  IloMismatchDExprSubstitution()
    : IloException("Can not mix pattern and index between declaration of dexpr and instanciation"), _expr(0), _indexes(), _values(){}
  void setInfo(IloExtractableI* extr, IloMapIndexArray indexes, IloMapIndexArray values){
	  _expr = extr;
	  _indexes = indexes;
	  _values = values;
  }
  IloExtractableI* getExtractable() const {return _expr;}
  IloMapIndexArray getIndexes() const {return _indexes;}
  IloMapIndexArray getValues() const {return _values;}
};
#endif

//----------------------------------------------------
class IloModelEvaluator {
private:
	IloModelEvaluatorI* _impl;
public:
	IloModelEvaluator():_impl(0) {}
	IloModelEvaluator(IloModelEvaluatorI* impl):_impl(impl) {}
	IloModelEvaluatorI* getImpl() const { return _impl; }

	IloModelEvaluator(IloEnv env);
	IloEnv getEnv() const;
	IloInt getValue(IloIntIndex idx) const;
	IloInt getValue(IloIntExprArg exp) const;
	IloInt getMin(IloIntExprArg exp) const;
	IloInt getMax(IloIntExprArg exp) const;
	IloNum getValue(IloNumIndex idx) const;
	IloNum getValue(IloNumExprArg exp) const;
	IloNum getLB(IloNumExprArg exp) const;
	IloNum getUB(IloNumExprArg exp) const;

	const char* getValue(IloSymbolIndex idx) const;
	const char* getValue(IloSymbolExprArg idx) const;

	/* <p> . </p> */
	IloIntCollection getValue(IloIntCollectionIndex idx) const;
	/* <p> . </p> */
	IloNumCollection getValue(IloNumCollectionIndex idx) const;
	/* <p> . </p> */
	IloAnyCollection getValue(IloSymbolCollectionIndex idx) const;

	IloTuple getValue(IloTupleIndex idx) const;
	IloTuple getValue(IloTuplePattern idx) const;
	IloTuple getValue(IloTupleExprArg exp) const;

	IloInt evaluate(IloIntExprArg exp) const;
	IloNum evaluate(IloNumExprArg exp) const;
    IloNumExprArg evaluateToNumExpr(IloNumExprArg exp) const;
	const char* evaluate(IloSymbolExprArg exp) const;
	//IloTuple evaluate(IloTupleExprArg exp) const;

	IloInt evaluate(IloIntIndex exp) const;
	IloNum evaluate(IloNumIndex exp) const;
	const char* evaluate(IloSymbolIndex exp) const;
	///** <p> . </p> */
	//IloTuple evaluate(IloTupleIndex exp) const;
	///** <p> . </p> */
	//IloTuple evaluate(IloTuplePattern exp) const;
	IloTuple evaluate(IloTuplePattern exp, IloTupleCollectionI* indexer) const;
	IloTuple evaluate(IloTupleIndex exp, IloTupleCollectionI* indexer) const;
	IloTuple evaluate(IloTupleExprArg exp, IloTupleCollectionI* indexer) const;

	IloTuple evaluateAsBuffer(IloTuplePattern exp, IloTupleCollectionI* indexer) const;
	IloTuplePattern evaluateAsConstPattern(IloTuplePattern exp) const;

	IloTuple evaluate(IloTuplePattern exp, IloTupleCollection indexer) const;
	IloTuple evaluate(IloTupleIndex exp, IloTupleCollection indexer) const;
	IloTuple evaluate(IloTupleExprArg exp, IloTupleCollection indexer) const;

	/* <p> . </p> */
	IloIntCollection evaluate(IloIntCollectionIndex idx) const;
	/* <p> . </p> */
	IloNumCollection evaluate(IloNumCollectionIndex idx) const;
	/* <p> . </p> */
	IloAnyCollection evaluate(IloSymbolCollectionIndex idx) const;
	/* <p> . </p> */
	IloIntCollection evaluate(IloIntCollectionExprArg idx) const;
	/* <p> . </p> */
	IloNumCollection evaluate(IloNumCollectionExprArg idx) const;
	/* <p> . </p> */
	IloAnyCollection evaluate(IloSymbolCollectionExprArg idx) const;

	IloTupleSet evaluate(IloTupleSetExprArg exp) const;

	/* <p> . </p> */
	IloInt evaluateAbsoluteIndex(IloMapI* m, IloMapExtractIndexI* idx) const;
	void evaluate(const IloMapIndexArray& input, IloMapIndexArray& output) const;
	IloMapIndex evaluate(const IloMapIndex& input, const IloTupleCollectionI* indexer = 0) const;

public:
	//-------------------------------------------------------------------------
	/// Iterator
	//-------------------------------------------------------------------------
	class Iterator {
	private:
    IloBool _ended;
		IloEvalIteratorI* _eval;
    Iterator(const Iterator&) {
      throw IloWrongUsage("IloModelEvaluator::Iterator Inaccessible constructor called.");
    }
	public:
		Iterator(IloModelEvaluatorI* e, IloComprehensionI* comp);
		Iterator(IloModelEvaluator, IloGenerator g1);
		Iterator(IloModelEvaluator, IloGeneratorArray generators);
		virtual ~Iterator();
		void start();
		void reset();
		void end();
		IloBool ok() const;
		void operator++();
		void display(ILOSTD(ostream)& out) const;
	};
public:
	//-------------------------------------------------------------------------
	/// Exceptions
	//-------------------------------------------------------------------------
	class Overflow : public IloException {
	public:
		Overflow():IloException("Evaluator had expression overflow.") {}
	};

	class Unbound : public IloException {
		IloEvalExprI* _exp;
	public:
		Unbound(IloEvalExprI* exp);
        IloExtractable getExpr() const;
		void print(ILOSTD(ostream)& out) const;
		virtual const char* getMessage() const;
	};

	class NotFoundTuple : public IloException {
		IloEvalTupleExprI* _exp;
    //IloTupleBuffer _buf;
		IloTupleCellArray _cells;
		IloTupleCollectionI* _coll;
	public:
		NotFoundTuple(IloEvalTupleExprI* exp, IloTupleCellArray cells, IloTupleCollectionI* coll);
		virtual void print(ILOSTD(ostream)& out) const;
		virtual const char* getMessage() const;
		IloTupleCellArray getCells() const{ return _cells; }
        IloTupleBuffer makeTupleBuffer() const { 
			IloTupleBuffer buf = new (_coll->getEnv()) IloTupleBufferI(_coll, _cells);
			return buf; 
		}
        IloTupleCollectionI* getTupleCollection() const { return (IloTupleCollectionI*)_coll; }
	};

	class UnboundSlicer : public IloException {
		IloEvalExprI* _exp;
	public:
		UnboundSlicer(IloEvalExprI* exp);
		void print(ILOSTD(ostream)& out) const;
		virtual const char* getMessage() const;
	};

	class ElementNumIndex : public IloException {
		const IloExtractableI* _ctx;
		const IloNumExprI* _idx;
	public:
		ElementNumIndex(const IloExtractableI* ctx, const IloNumExprI* e);
		void print(ILOSTD(ostream)& out) const;
		virtual const char* getMessage() const;
	};

	class AggregateSetBody : public IloException {
	private:
		const IloExtractableI* _index;
		const IloExtractableI* _context;
	public:
		AggregateSetBody(const IloExtractableI* index);
		void print(ILOSTD(ostream)& out) const;
		virtual const char* getMessage() const;
		void setContext(const IloExtractableI* c) { _context = c; }
	};

public:
	enum ConstraintStatus {
		Violated,
		Satisfied,
		Unknown
	};

  enum ErrorStatus {
    None,
    OverflowOccured
  };
};

ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out,
							const IloModelEvaluator::Iterator& p);

IloConstraint IloConditionalConstraint(IloEnv env, IloIntExprArg expr,
                                       IloConstraint,IloConstraint, const char* name = 0);

#endif
