// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloexpressioniH
#define __CONCERT_iloexpressioniH

#if !defined(__CONCERT_iloexpressionH)
# include <ilconcert/iloexpression.h>
#endif

///////////////////////////////////////////
//
// IloNumVar changes
//

class IloSetNumVarBounds : public IloChange {
  ILOCHANGEDECL

  IloNumVarI* _var;
  IloNum    _lb, _ub;

  IloSetNumVarBounds();
public:
  IloSetNumVarBounds(IloNumVarI* var, IloNum lb, IloNum ub)
    : _var(var), _lb(lb), _ub(ub)   {}

  IloNumVarI* getVar() const    { return (_var); }
  IloNum    getLb()  const    { return (_lb); }
  IloNum    getUb()  const    { return (_ub); }
};

///////////////////////////////////////////
//
// IloNumVarArray changes
//

class IloSetNumVarArrayBounds : public IloChange {
  ILOCHANGEDECL

  const IloNumVarArray _var;
  const IloNumArray  _lb;
  const IloNumArray  _ub;

  IloSetNumVarArrayBounds (const IloNumVarArray var,
                           const IloNumArray    lb,
                           const IloNumArray    ub)
    : _var(var), _lb(lb), _ub(ub) {}

  friend class IloNumVarArray;
  
public:
  const IloNumVarArray& getVars() const { return (_var); }
  const IloNumArray&  getLbs()  const { return (_lb); }
  const IloNumArray&  getUbs()  const { return (_ub); }
};


///////////////////////////////////////////////////////
//
// IloSemiContVar
//
///////////////////////////////////////////////////////

class IloSemiContVarI : public IloNumVarI {
  ILOEXTRMEMBERS1DECL(IloSemiContVarI, IloNumVarI,
                      IloNum,          _threshold)
  /* use methods inherited from IloNumVarI
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  virtual void atRemove(IloExtractableI* user=0, IloAny info = 0);
  */
public:
  IloSemiContVarI(IloEnvI*         env,
                  IloNum           sclb,
                  IloNum           ub,
                  IloNumVar::Type  type = ILOFLOAT,
                  const char*      name = 0)
    : IloNumVarI(env, type, 0.0, ub, name), _threshold(sclb) {}

  IloNum getBd() const                  { return (_threshold); }
  void   setBd(IloNum threshold);
};

class IloSetSemiContBound : public IloChange {
  ILOCHANGEDECL
  friend class IloSemiContVarI;

  IloSemiContVarI* _var;
  IloNum           _oldBd;
  IloNum           _newBd;

  IloSetSemiContBound(IloSemiContVarI* var, IloNum oldBd, IloNum newBd)
    : _var(var), _oldBd(oldBd), _newBd(newBd) {}

public:

  IloSemiContVarI* getVar() const   { return (_var); }
  IloNum           getOldBd() const { return (_oldBd); }
  IloNum           getNewBd() const { return (_newBd); }
};


/////////////////////////////////////////////////////////////////////////
//
// IloConstraintIndVarI
//

class IloConstraintIndVarI : public IloNumVarI {
  ILOEXTRDECL
  IloConstraintI* _constraint;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloConstraintIndVarI(IloEnvI* env, 
		       IloConstraintI* constraint, 
		       const char* name)
    : IloNumVarI(env, ILOINT, 0, 1, name), _constraint(constraint) {}

  ILOEXTROTHERDECL

  IloConstraintI* getConstraint() const {return _constraint;}
};

/////////////////////////////////////////////////////////////////////////
//
// IloNumVar Changes
//
/////////////////////////////////////////////////////////////////////////

class IloSetNumVarPossibleValues : public IloChange {
  ILOCHANGEDECL
  
  IloNumVarI* _var;
  IloNumArray _floatValues;
  IloIntArray _intValues;
  
  IloSetNumVarPossibleValues();
public:
  IloSetNumVarPossibleValues(IloNumVarI* var,
			     const IloNumArray values)
    : _var(var), _floatValues(values), _intValues(0)   {}
  IloSetNumVarPossibleValues(IloNumVarI* var,
			     const IloIntArray values)
    : _var(var), _floatValues(0), _intValues(values)   {}
  
  IloNumVarI* getVar() const     { return (_var); }
  IloNumArray  getFloatValues()  const { return (_floatValues); }
  IloIntArray  getIntValues()  const { return (_intValues); }
};


class IloNumLinExprTermI {
  IloNumVarI* _var;
  IloNumLinExprTermI* _next;
  IloNum _coef;
public:
  IloNumLinExprTermI(IloNumVarI* var, IloNum coef) 
  { 
    _var = var; 
    _coef = coef;
    _next = 0;
  }
  IloNum getCoef() const { return _coef; }
  void setCoef(IloNum coef)  { _coef = coef; }
  IloNumVarI* getVar() const { return _var; }
  IloNumLinExprTermI* getNext() const { return _next; }
  void setNext(IloNumLinExprTermI* next) { _next = next; }
  IloEnvI* getEnv() const { return _var->getEnv(); }
  friend class IloNumLinTermI;
  friend class IloIntLinTermI;
  friend class IloRangeI;
  friend class IloEnvI;
};


class IloNumLinTermI : public IloNumExprI {
public:
  class ExprList {
    IloNumExprI* _expr;
    ExprList* _next;
  public:
    ExprList(IloNumExprI* expr, ExprList* next) : _expr(expr), _next(next) {}
    IloNumExprI* getExpr() const { return _expr; }
    void setNext(ExprList* next) { _next = next; }
    ExprList* getNext() const { return _next; }
    friend class IloIntLinTermI;
    friend class IloNumLinTermI;
  };
private:

  ILOEXTRDECL
  IloNumLinExprTermI* _firstLinear;
  IloNumLinExprTermI* _lastLinear;
  IloNum _cst;
  ExprList* _firstGeneral;
  ExprList* _lastGeneral;
  IloBool   _isNormalized;
  virtual IloNumExprI* copy();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
protected:
  virtual void atRemove(IloExtractableI* user=0, IloAny info = 0);
public:
  IloNumLinTermI(const IloNumLinExprTerm term);
  IloNumLinTermI(const IloIntLinExprTerm term);
  IloNumLinTermI(IloEnvI* env, IloNum cst);
  virtual ~IloNumLinTermI();
  IloNumLinExprTermI* getFirstLinear() const { return _firstLinear; }
  IloNumLinExprTermI* getLastLinear() const { return _lastLinear; }
  ExprList* getFirstGeneral() const { return _firstGeneral; }
  ExprList* getLastGeneral() const { return _lastGeneral; }
  void append(IloNumLinExprTermI* ap) {
    if (_lastLinear) {
      _lastLinear->_next = ap;
      _lastLinear = ap;
    }
    else {
      _firstLinear = ap;
      _lastLinear = ap;
    }
  }
  void prepend(IloNumLinExprTermI*);
  void append(IloNumExprI*);
  void append(IloNumLinTermI*);
  void setNumConstant(IloNum constant) { _cst = constant; }
  IloNum getNumConstant() const { return _cst; }
  ExprList* getGeneral() const { return _firstGeneral; }
  virtual IloBool displayTerm(ILOSTD(ostream)&, IloNumLinExprTermI*) const;
  virtual void display(ILOSTD(ostream)&) const;
  IloNumLinTermI* multiply(IloNum constant);
  IloNumLinTermI* divide(IloNum constant);
  friend class IloIntLinTermI;
  friend class IloExpr;
  friend class IloDefaultDeleterExtensionI;
  IloNum setCoef(const IloNumVar var, const IloNum val);
  void setCoef(const IloNumVarArray vars, const IloNumArray vals,
               IloNumArray status = 0);
  void remove(IloNumVarI* var);
  void remove(const IloExtractableArray vars);
  void removeMarked();
  IloInt normalize();
  virtual IloNumLinTermI* toNumLinTermI();
  virtual IloBool isInteger() const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  void clear();
  IloBool isLinear() const;
};

class IloNumLinTermRemoveVar : public IloChange {
  ILOCHANGEDECL

  IloNumLinTermI*        _rng;
  const IloNumVarI* _var;
  
public:
  IloNumLinTermRemoveVar(IloNumLinTermI* rng, IloNumVarI* var)
    : _rng(rng), _var(var)    {}

  const IloNumLinTermI*  getNumLinTerm() const   { return (_rng); }
  const IloNumVarI* getVar   () const   { return (_var); }
};

class IloNumLinTermAddVar : public IloChange {
  ILOCHANGEDECL

  IloNumLinTermI*        _rng;
  const IloNumVarI* _var;
  
public:
  IloNumLinTermAddVar(IloNumLinTermI* rng, IloNumVarI* var)
    : _rng(rng), _var(var)    {}

  const IloNumLinTermI*  getNumLinTerm() const   { return (_rng); }
  const IloNumVarI* getVar   () const   { return (_var); }
};

class IloIntLinTermI : public IloIntExprI {
public:
  class ExprList {
    IloIntExprI* _expr;
    ExprList* _next;
  public:
    ExprList(IloIntExprI* expr, ExprList* next) : _expr(expr), _next(next) {}
    IloIntExprI* getExpr() const { return _expr; }
    void setNext(ExprList* next) { _next = next; }
    ExprList* getNext() const { return _next; }
    friend class IloIntLinTermI;
  };
private:
  ILOEXTRDECL
  IloNumLinExprTermI* _firstLinear;
  IloNumLinExprTermI* _lastLinear;
  IloInt _cst;
  ExprList* _firstGeneral;
  ExprList* _lastGeneral;
  virtual IloIntExprI* intCopy();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
protected:
  virtual void atRemove(IloExtractableI* user=0, IloAny info = 0);
public:
  IloIntLinTermI(const IloIntLinExprTerm term);
  IloIntLinTermI(IloEnvI* env, IloInt cst);
  virtual ~IloIntLinTermI();
  IloNumLinExprTermI* getFirstLinear() const { return _firstLinear; }
  IloNumLinExprTermI* getLastLinear() const { return _lastLinear; }
  ExprList* getFirstGeneral() const { return _firstGeneral; }
  void append(IloNumLinExprTermI*);
  void prepend(IloNumLinExprTermI*);
  void append(IloIntExprI*);
  void append(IloIntLinTermI*);
  virtual void display(ILOSTD(ostream)&) const;
  virtual IloBool displayTerm(ILOSTD(ostream)&, IloNumLinExprTermI*) const;
  IloNum getNumConstant() const { return _cst; }
  void setIntConstant(IloInt constant) { _cst = constant; }
  IloInt getIntConstant() const { return _cst; }
  virtual IloNumLinTermI* toNumLinTermI();
  virtual IloIntLinTermI* toIntLinTermI();
  IloIntLinTermI* multiply(IloInt constant);
  virtual IloNum eval(const IloAlgorithm alg) const;
  void remove(const IloExtractableArray vars);
  void remove(IloIntVarI* var);
  void removeMarked();
  IloExtractableI* makeClone(IloEnvI*) const;
  friend class IloDefaultDeleterExtensionI;
};

class IloIntLinTermRemoveVar : public IloChange {
  ILOCHANGEDECL

  IloIntLinTermI*        _rng;
  const IloIntVarI* _var;
  
public:
  IloIntLinTermRemoveVar(IloIntLinTermI* rng, IloIntVarI* var)
    : _rng(rng), _var(var)    {}

  const IloIntLinTermI*  getIntLinTerm() const   { return (_rng); }
  const IloIntVarI* getVar   () const   { return (_var); }
};

class IloIntModuloI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _left;
  IloIntExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntModuloI(IloEnvI* env, IloIntExprI* left, IloIntExprI* right) : 
    IloIntExprI(env), _left(left->intLockExpr()), 
    _right(right->intLockExpr()) {}
  virtual ~IloIntModuloI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getLeftExpr() const { return _left; }
  IloIntExprI* getRightExpr() const { return _right; }
};

class IloIntModuloCstI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  IloInt _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntModuloCstI(IloEnvI* env, IloIntExprI* arg, IloInt cst) : 
    IloIntExprI(env), _arg(arg->intLockExpr()), _cst(cst) {}
  virtual ~IloIntModuloCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _arg; }
  IloInt getIntCst() const { return _cst; }
};


class IloIntRecModuloCstI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  IloInt _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntRecModuloCstI(IloEnvI* env, IloInt cst, IloIntExprI* arg) : 
    IloIntExprI(env), _arg(arg->intLockExpr()), _cst(cst) {}
  virtual ~IloIntRecModuloCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _arg; }
  IloInt getIntCst() const { return _cst; }
};
//

class IloNumTimesCstI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  IloNum _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumTimesCstI(IloEnvI* env, IloNumExprI* arg, IloNum cst) : 
    IloNumExprI(env), _arg(arg->lockExpr()), _cst(cst) {}
  virtual ~IloNumTimesCstI();
  void display(ILOSTD(ostream)& out) const;
  IloNumExprI* getExpr() const { return _arg; }
  IloNum getNumCst() const { return _cst; }
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
};

class IloNumTimesI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _left;
  IloNumExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumTimesI(IloEnvI* env, IloNumExprI* left, IloNumExprI* right) : 
    IloNumExprI(env), _left(left->lockExpr()), _right(right->lockExpr()) {}
  virtual ~IloNumTimesI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumExprI* getLeftExpr() const { return _left; }
  IloNumExprI* getRightExpr() const { return _right; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
};

class IloNumDivI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _left;
  IloNumExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumDivI(IloEnvI* env, IloNumExprI* left, IloNumExprI* right) : 
    IloNumExprI(env), _left(left->lockExpr()), _right(right->lockExpr()) {}
  virtual ~IloNumDivI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
  IloNumExprI* getLeftExpr() const { return _left; }
  IloNumExprI* getRightExpr() const { return _right; }
};

class IloNumDivCstI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  IloNum _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumDivCstI(IloEnvI* env, IloNumExprI* arg, IloNum cst) : 
    IloNumExprI(env), _arg(arg->lockExpr()), _cst(cst) {}
  virtual ~IloNumDivCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
  IloNumExprI* getExpr() const { return _arg; }
  IloNum getNumCst() const { return _cst; }
};

class IloNumRecDivCstI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  IloNum _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumRecDivCstI(IloEnvI* env, IloNumExprI* arg, IloNum cst) : 
    IloNumExprI(env), _arg(arg->lockExpr()), _cst(cst) {}
  virtual ~IloNumRecDivCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
  IloNumExprI* getExpr() const { return _arg; }
  IloNum getNumCst() const { return _cst; }
};

class IloIntTimesCstI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  IloInt _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntTimesCstI(IloEnvI* env, IloIntExprI* arg, IloInt cst) : 
    IloIntExprI(env), _arg(arg->intLockExpr()), _cst(cst) {}
  virtual ~IloIntTimesCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloIntExprI* getExpr() const { return _arg; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloInt getIntCst() const { return _cst; }
};

class IloIntTimesI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _left;
  IloIntExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntTimesI(IloEnvI* env, IloIntExprI* left, IloIntExprI* right) : 
    IloIntExprI(env), _left(left->intLockExpr()), 
    _right(right->intLockExpr()) {}
  virtual ~IloIntTimesI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloIntExprI* getLeftExpr() const { return _left; }
  IloIntExprI* getRightExpr() const { return _right; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
};

class IloIntDivI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _left;
  IloIntExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntDivI(IloEnvI* env, IloIntExprI* left, IloIntExprI* right) : 
    IloIntExprI(env), _left(left->intLockExpr()), 
    _right(right->intLockExpr()) {}
  virtual ~IloIntDivI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getLeftExpr() const { return _left; }
  IloIntExprI* getRightExpr() const { return _right; }
};

class IloIntDivCstI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  IloInt _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntDivCstI(IloEnvI* env, IloIntExprI* arg, IloInt cst) : 
    IloIntExprI(env), _arg(arg->intLockExpr()), _cst(cst) {}
  virtual ~IloIntDivCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _arg; }
  IloInt getIntCst() const { return _cst; }
};

class IloIntRecDivCstI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  IloInt _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntRecDivCstI(IloEnvI* env, IloIntExprI* arg, IloInt cst) : 
    IloIntExprI(env), _arg(arg->intLockExpr()), _cst(cst) {}
  virtual ~IloIntRecDivCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _arg; }
  IloInt getIntCst() const { return _cst; }
};

//////////////////////////////////////////////////////////////////
// element expressions
//////////////////////////////////////////////////////////////////

class IloNumExprArrayElementI : public IloNumExprI {
  ILOEXTRDECL
private:
  IloIntExprI*    _index;
  IloNumExprArray _array;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumExprArrayElementI(IloEnvI* env, 
			  IloIntExprI* idx, IloNumExprArray array);
  virtual ~IloNumExprArrayElementI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumExprArray getExprArray() const { return _array; }
  IloIntExprI* getIndex() const { return _index; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
};

class IloIntExprArrayElementI : public IloIntExprI {
  ILOEXTRDECL
private:
  IloIntExprI*    _index;
  IloIntExprArray _array;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntExprArrayElementI(IloEnvI* env, IloIntExprI* idx,
			  IloIntExprArray array);
  virtual ~IloIntExprArrayElementI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloIntExprArray getExprArray() const { return _array; }
  IloIntExprI* getIndex() const { return _index; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
};

class IloIntArrayElementI : public IloIntExprI {
  ILOEXTRDECL
private:
  IloIntExprI* _index;
  IloIntArray  _array;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntArrayElementI(IloEnvI* env, IloIntExprI* index, IloIntArray array):
    IloIntExprI(env), _index(index->intLockExpr()), _array(array) {}
  virtual ~IloIntArrayElementI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloIntArray getArray() const { return _array; }
  IloIntExprI* getIndex() const { return _index; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
};

class IloNumArrayElementI : public IloNumExprI {
  ILOEXTRDECL
private:
  IloIntExprI* _index;
  IloNumArray  _array;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumArrayElementI(IloEnvI* env, IloIntExprI* index, IloNumArray array):
    IloNumExprI(env), _index(index->intLockExpr()), _array(array) {}
  virtual ~IloNumArrayElementI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumArray getArray() const { return _array; }
  IloIntExprI* getIndex() const { return _index; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
};

//////////////////////////////////////////////////////////////////
// general arithmetic
//////////////////////////////////////////////////////////////////

class IloSinI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloSinI(IloEnvI* env, IloNumExprI* arg) : IloNumExprI(env), 
					    _arg(arg->lockExpr()) {}
  virtual ~IloSinI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};

class IloCosI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloCosI(IloEnvI* env, IloNumExprI* arg) : IloNumExprI(env), 
					    _arg(arg->lockExpr()) {}
  virtual ~IloCosI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};

class IloTanI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTanI(IloEnvI* env, IloNumExprI* arg) : IloNumExprI(env), 
					    _arg(arg->lockExpr()) {}
  virtual ~IloTanI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};

class IloArcSinI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloArcSinI(IloEnvI* env, IloNumExprI* arg) : IloNumExprI(env), 
					       _arg(arg->lockExpr()) {}
  virtual ~IloArcSinI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};

class IloArcCosI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloArcCosI(IloEnvI* env, IloNumExprI* arg) : IloNumExprI(env), 
					       _arg(arg->lockExpr()) {}
  virtual ~IloArcCosI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};
class IloArcTanI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloArcTanI(IloEnvI* env, IloNumExprI* arg) : IloNumExprI(env), 
					       _arg(arg->lockExpr()) {}
  virtual ~IloArcTanI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};
class IloExponentI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloExponentI(IloEnvI* env, IloNumExprI* arg) : IloNumExprI(env), 
						 _arg(arg->lockExpr()) {}
  virtual ~IloExponentI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};
class IloLogI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloLogI(IloEnvI* env, IloNumExprI* arg) : IloNumExprI(env), 
					    _arg(arg->lockExpr()) {}
  virtual ~IloLogI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};

class IloIntSquareI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntSquareI(IloEnvI* env, IloIntExprI* arg) : 
    IloIntExprI(env), _arg(arg->intLockExpr()) {}
  virtual ~IloIntSquareI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _arg; }
};

class IloNumSquareI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumSquareI(IloEnvI* env, IloNumExprI* arg) : 
    IloNumExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloNumSquareI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  virtual IloBool isInteger() const;
};

class IloIntAbsI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntAbsI(IloEnvI* env, IloIntExprI* arg) : 
    IloIntExprI(env), _arg(arg->intLockExpr()) {}
  virtual ~IloIntAbsI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _arg; }
};

class IloNumAbsI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumAbsI(IloEnvI* env, IloNumExprI* arg) : 
    IloNumExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloNumAbsI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  virtual IloBool isInteger() const;
};

class IloSgnI : public IloIntExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloSgnI(IloEnvI* env, IloNumExprI* arg) : 
    IloIntExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloSgnI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};

////////////////////////////////////////////////////////////////////
// IloMin
////////////////////////////////////////////////////////////////////

class IloNumMinCstI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  IloNum _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumMinCstI(IloEnvI* env, IloNumExprI* arg, IloNum cst) : 
    IloNumExprI(env), _arg(arg->lockExpr()), _cst(cst) {}
  virtual ~IloNumMinCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  IloNum getNumCst() const { return _cst; }
  virtual IloBool isInteger() const;
};

class IloNumMinI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _left;
  IloNumExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumMinI(IloEnvI* env, IloNumExprI* left, IloNumExprI* right) : 
    IloNumExprI(env), _left(left->lockExpr()), _right(right->lockExpr()) {}
  virtual ~IloNumMinI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getLeftExpr() const { return _left; }
  IloNumExprI* getRightExpr() const { return _right; }
  virtual IloBool isInteger() const;
};

class IloIntMinCstI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  IloInt _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntMinCstI(IloEnvI* env, IloIntExprI* arg, IloInt cst) : 
    IloIntExprI(env), _arg(arg->intLockExpr()), _cst(cst) {}
  virtual ~IloIntMinCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _arg; }
  IloInt getIntCst() const { return _cst; }
};

class IloIntMinI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _left;
  IloIntExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntMinI(IloEnvI* env, IloIntExprI* left, IloIntExprI* right) : 
    IloIntExprI(env), _left(left->intLockExpr()), 
    _right(right->intLockExpr()) {}
  virtual ~IloIntMinI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getLeftExpr() const { return _left; }
  IloIntExprI* getRightExpr() const { return _right; }
};

class IloNumMinArrayI  : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprArray _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumMinArrayI(IloEnvI* env, IloNumExprArray arg);
  virtual ~IloNumMinArrayI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumExprArray getExprArray() const { return _arg; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
};

class IloIntMinArrayI  : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprArray _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntMinArrayI(IloEnvI* env, IloIntExprArray arg);
  virtual ~IloIntMinArrayI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloIntExprArray getExprArray() const { return _arg; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
};

////////////////////////////////////////////////////////////////////
// IloMax
////////////////////////////////////////////////////////////////////

class IloNumMaxCstI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  IloNum _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumMaxCstI(IloEnvI* env, IloNumExprI* arg, IloNum cst) : 
    IloNumExprI(env), _arg(arg->lockExpr()), _cst(cst) {}
  virtual ~IloNumMaxCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  IloNum getNumCst() const { return _cst; }
  virtual IloBool isInteger() const;
};

class IloNumMaxI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _left;
  IloNumExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumMaxI(IloEnvI* env, IloNumExprI* left, IloNumExprI* right) : 
    IloNumExprI(env), _left(left->lockExpr()), _right(right->lockExpr()) {}
  virtual ~IloNumMaxI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getLeftExpr() const { return _left; }
  IloNumExprI* getRightExpr() const { return _right; }
  virtual IloBool isInteger() const;
};


class IloIntMaxCstI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _arg;
  IloInt _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntMaxCstI(IloEnvI* env, IloIntExprI* arg, IloInt cst) : 
    IloIntExprI(env), _arg(arg->intLockExpr()), _cst(cst) {}
  virtual ~IloIntMaxCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _arg; }
  IloInt getIntCst() const { return _cst; }
};

class IloIntMaxI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _left;
  IloIntExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntMaxI(IloEnvI* env, IloIntExprI* left, IloIntExprI* right) : 
    IloIntExprI(env), _left(left->intLockExpr()), 
    _right(right->intLockExpr()) {}
  virtual ~IloIntMaxI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getLeftExpr() const { return _left; }
  IloIntExprI* getRightExpr() const { return _right; }
};

class IloNumMaxArrayI  : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprArray _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumMaxArrayI(IloEnvI* env, IloNumExprArray arg);
  virtual ~IloNumMaxArrayI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumExprArray getExprArray() const { return _arg; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
};

class IloIntMaxArrayI  : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprArray _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntMaxArrayI(IloEnvI* env, IloIntExprArray arg);
  virtual ~IloIntMaxArrayI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloIntExprArray getExprArray() const { return _arg; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
};

//////////////////////////////////////////////////////////////////////////
// IloPower
//////////////////////////////////////////////////////////////////////////

class IloNumPowerI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _left;
  IloNumExprI* _right;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumPowerI(IloEnvI* env, IloNumExprI* left, IloNumExprI* right) : 
    IloNumExprI(env), _left(left->lockExpr()), _right(right->lockExpr()) {}
  virtual ~IloNumPowerI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
  IloNumExprI* getLeftExpr() const { return _left; }
  IloNumExprI* getRightExpr() const { return _right; }
};

class IloNumPowerCstI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  IloNum _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumPowerCstI(IloEnvI* env, IloNumExprI* arg, IloNum cst) : 
    IloNumExprI(env), _arg(arg->lockExpr()), _cst(cst) {}
  virtual ~IloNumPowerCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
  IloNumExprI* getExpr() const { return _arg; }
  IloNum getNumCst() const { return _cst; }
};

class IloNumRecPowerCstI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  IloNum _cst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumRecPowerCstI(IloEnvI* env, IloNumExprI* arg, IloNum cst) : 
    IloNumExprI(env), _arg(arg->lockExpr()), _cst(cst) {}
  virtual ~IloNumRecPowerCstI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloBool isInteger() const;
  IloNumExprI* getExpr() const { return _arg; }
  IloNum getNumCst() const { return _cst; }
};

class IloExprPiecewiseLinearI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _term;
  IloBool      _useSlope; 
  IloNumArray  _point;
  IloNumArray  _slope;
  IloNum       _a;
  IloNum       _b;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloExprPiecewiseLinearI(IloEnvI* env, 
			  IloBool useSlope, 
                          IloNumExprI* x,
                          const IloNumArray point,
                          const IloNumArray slope,
                          IloNum a,
                          IloNum b);
  IloExprPiecewiseLinearI(IloEnvI* env,		
			  IloNumExprI* x,
                          const IloNumArray point,
                          const IloNumArray slope,
                          IloNum a,
                          IloNum fa);
  IloExprPiecewiseLinearI(IloEnvI* env, 
			  IloNumExprI* x,
                          IloNum firstSlope, 
                          const IloNumArray point,
                          const IloNumArray value,
                          IloNum lastSlope);
  
  virtual ~IloExprPiecewiseLinearI();
  //  virtual void destroy(); //CHECKME

  //  virtual IloNumExprI* getCopy() const; //CHECKME
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;

  IloNum getPiecewiseLinearValue(IloNum) const;

  IloBool useSlope() const { return _useSlope; } 
  IloNumArray getPoint() const { return _point; }
  IloNumArray getSlope() const { return _slope; }
  IloNum getA() const { return _a; }
  IloNum getB() const { return _b; } 
  IloNumExprI* getTerm() const { return _term; }
};

class IloMonotonicNumExprI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _term;
  IloNumFunction _f;
  IloNumFunction _invf;
  IloBool _isDecreasing; // if false, assume increasing
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloMonotonicNumExprI(IloEnvI* env, 
		       IloNumExprI* term,
		       IloNumFunction f,
		       IloNumFunction invf,
		       IloBool isDecreasing)
    :IloNumExprI(env), 
     _term(term->lockExpr()), 
     _f(f), 
     _invf(invf), 
     _isDecreasing(isDecreasing){}
  virtual ~IloMonotonicNumExprI();

  IloBool isDecreasing() const {
    return _isDecreasing;
  }
  IloNumFunction getF() const {
    return _f;
  }
  IloNumFunction getInvF() const {
    return _invf;
  }
  IloNumExprI* getTerm() const { 
    return _term; 
  }
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
};



class IloNumCeilI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumCeilI(IloEnvI* env, IloNumExprI* arg) : 
    IloNumExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloNumCeilI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  virtual IloBool isInteger() const;
};

class IloNumFloorI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumFloorI(IloEnvI* env, IloNumExprI* arg) : 
    IloNumExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloNumFloorI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  virtual IloBool isInteger() const;
};

class IloNumTruncI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumTruncI(IloEnvI* env, IloNumExprI* arg) : 
    IloNumExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloNumTruncI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  virtual IloBool isInteger() const;
};

class IloDistToIntI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloDistToIntI(IloEnvI* env, IloNumExprI* arg) : 
    IloNumExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloDistToIntI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  virtual IloBool isInteger() const;
};

class IloNumToIntI : public IloIntExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumToIntI(IloEnvI* env, IloNumExprI* arg) : 
    IloIntExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloNumToIntI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
};


class IloNumFractI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumFractI(IloEnvI* env, IloNumExprI* arg) : 
    IloNumExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloNumFractI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  virtual IloBool isInteger() const;
};

class IloNumRoundI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _arg;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumRoundI(IloEnvI* env, IloNumExprI* arg) : 
    IloNumExprI(env), _arg(arg->lockExpr()) {}
  virtual ~IloNumRoundI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _arg; }
  virtual IloBool isInteger() const;
};

//-------------------------------------------------------------
class IloConditionalIntExprI: public IloIntExprI {
	ILOEXTRDECL
	IloConstraintI* _cond;
	IloIntExprI* _then;
	IloIntExprI* _else;
	IloInt _thenValue;
	IloInt _elseValue;

public:
	IloConditionalIntExprI(IloEnvI* env, IloConstraintI* cond, IloIntExprI* left, IloIntExprI* right ) 
		:IloIntExprI(env), _cond((IloConstraintI*)cond->lockExpr()), _then((IloIntExprI*)left->lockExpr()), _else((IloIntExprI*)right->lockExpr()), _thenValue(0), _elseValue(0) {
    }
		
	IloConditionalIntExprI(IloEnvI* env, IloConstraintI* cond, IloIntExprI* left, IloInt right ) 
		:IloIntExprI(env), _cond((IloConstraintI*)cond->lockExpr()), _then((IloIntExprI*)left->lockExpr()), _else(0), _thenValue(0), _elseValue(right) {
    }

   	IloConditionalIntExprI(IloEnvI* env, IloConstraintI* cond, IloInt left, IloIntExprI* right ) 
		:IloIntExprI(env), _cond((IloConstraintI*)cond->lockExpr()), _then(0), _else((IloIntExprI*)right->lockExpr()), _thenValue(left), _elseValue(0) {
    }

   	IloConditionalIntExprI(IloEnvI* env, IloConstraintI* cond, IloInt left, IloInt right ) 
		:IloIntExprI(env), _cond((IloConstraintI*)cond->lockExpr()), _then(0), _else(0), _thenValue(left), _elseValue(right) {
    }

    ~IloConditionalIntExprI();

    void display(ILOSTD(ostream)& out) const;
    virtual IloNum eval(const IloAlgorithm alg) const;
    virtual IloExtractableI* makeClone(IloEnvI*) const;
    virtual IloBool isInteger() const;
    virtual void visitSubExtractables(IloExtractableVisitor* v);

    IloConstraintI* getCond() const {return _cond;}
	  IloIntExprI* getThen() const {return _then;}
	  IloIntExprI* getElse() const {return _else;}
    IloInt getThenValue() const {return _thenValue;}
    IloInt getElseValue() const {return _elseValue;}
};

//-------------------------------------------------------------
class IloConditionalNumExprI: public IloNumExprI {
	ILOEXTRDECL
	IloConstraintI* _cond;
	IloNumExprI* _then;
	IloNumExprI* _else;
	IloNum _thenValue;
	IloNum _elseValue;

public:
	IloConditionalNumExprI(IloEnvI* env, IloConstraintI* cond, IloNumExprI* left, IloNumExprI* right ) 
		:IloNumExprI(env), _cond((IloConstraintI*)cond->lockExpr()), _then((IloNumExprI*)left->lockExpr()), _else((IloNumExprI*)right->lockExpr()), _thenValue(0), _elseValue(0) {
    }
		
	IloConditionalNumExprI(IloEnvI* env, IloConstraintI* cond, IloNumExprI* left, IloNum right ) 
		:IloNumExprI(env), _cond((IloConstraintI*)cond->lockExpr()), _then((IloNumExprI*)left->lockExpr()), _else(0), _thenValue(0), _elseValue(right) {
    }

   	IloConditionalNumExprI(IloEnvI* env, IloConstraintI* cond, IloNum left, IloNumExprI* right ) 
		:IloNumExprI(env), _cond((IloConstraintI*)cond->lockExpr()), _then(0), _else((IloNumExprI*)right->lockExpr()), _thenValue(left), _elseValue(0) {
    }

   	IloConditionalNumExprI(IloEnvI* env, IloConstraintI* cond, IloNum left, IloNum right ) 
		:IloNumExprI(env), _cond((IloConstraintI*)cond->lockExpr()), _then(0), _else(0), _thenValue(left), _elseValue(right) {
    }

    ~IloConditionalNumExprI();

    void display(ILOSTD(ostream)& out) const;
    virtual IloNum eval(const IloAlgorithm alg) const;
    virtual IloExtractableI* makeClone(IloEnvI*) const;
    virtual IloBool isInteger() const;
    virtual void visitSubExtractables(IloExtractableVisitor* v);

    IloConstraintI* getCond() const {return _cond;}
	  IloNumExprI* getThen() const {return _then;}
	  IloNumExprI* getElse() const {return _else;}
    IloNum getThenValue() const {return _thenValue;}
    IloNum getElseValue() const {return _elseValue;}
};

//-------------------------------------------------------------
class IloCountI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprArray _arg;
  IloInt          _value;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloCountI(IloEnvI* env, IloIntExprArray exprs, IloInt value);

  virtual ~IloCountI();
  //  virtual void destroy(); //CHECKME

  IloIntExprArray getExprArray() const { return _arg; }
  IloInt          getValue() const     { return _value; }

  //  virtual IloNumExprI* getCopy() const; //CHECKME
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
};

//-------------------------------------------------------------
class IloStandardDeviationI : public IloNumExprI {
  ILOEXTRDECL
  IloIntExprArray _arg;
  IloNum          _meanLB;
  IloNum          _meanUB;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloStandardDeviationI(IloEnvI* env,
                        IloIntExprArray exprs,
                        IloNum meanLB,
                        IloNum meanUB);
  virtual ~IloStandardDeviationI();
  IloIntExprArray getExprArray() const { return _arg; }
  IloNum getMeanLB() const { return _meanLB; }
  IloNum getMeanUB() const { return _meanUB; }
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  static IloNum Eval(const IloAlgorithm alg, const IloIntExprArray arg);
  virtual IloExtractableI* makeClone(IloEnvI*) const;
};

#endif
