// --------------------------------------------------*- C++ -*---------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilooplcpiH
#define __CONCERT_ilooplcpiH

#if !defined(__CONCERT_ilomodelH)
# include <ilconcert/ilomodel.h>
#endif

#if !defined(__CONCERT_ilomapH)
# include <ilconcert/ilomap.h>
#endif

#if !defined(__CONCERT_iloevaluatorH)
# include <ilconcert/iloevaluator.h>
#endif

/////////////////////////////////////
// IloOplIntExpr
//
class IloOplIntExpr {
public:
  enum Type {
    Invalid  =0,
    IntConst =1,
    IntExpr  =2,
    IntIndex =3
  };
private:
  Type   _type;
  union {
    IloInt _int;
    IloIntExprI* _intExpr;
  } _data;
  IloOplIntExpr(Type t, IloIntExprI* x) :_type(t) { _data._intExpr = x;}
public:
  IloOplIntExpr();
  IloOplIntExpr(const IloOplIntExpr& i);
  IloOplIntExpr(IloIntExprArg x);
  IloOplIntExpr(IloIntVar x);
  IloOplIntExpr(IloIntIndex x);
  IloOplIntExpr(IloInt x);
  ~IloOplIntExpr() {}

  Type getType() const { return _type; }

  IloOplIntExpr getClone(IloEnvI* env) const;
  void operator= (const IloOplIntExpr& i);
  IloBool isValid() const { return !(Invalid == _type); }
  IloBool isConstant() const { return !(_type & 2); }
  IloBool isExtractable() const { return (_type & 2); }
  IloBool isIndex() const { return (IntIndex == _type); }

  IloInt asInt() const {
    IloAssert(_type == IntConst, "OPL int expr Item is not an integer.");
    return _data._int;
  }
  void getValue(IloInt& res) { res = asInt(); }
  IloIntExprI* asIntExpr() const {
    IloAssert(isExtractable(), "OPL int expr is not an integer expression.");
    return (IloIntExprI*)_data._intExpr;
  }
  void getValue(IloIntExpr& res) { res = asIntExpr(); }
  void display(ILOSTD(ostream)& out) const;
  void accept(IloExtractableI* parent,IloExtractableVisitor* v);
  static Type GetType(IloInt) { return IntConst; }
  static Type GetType(IloIntExprArg) { return IntExpr; }
  static Type GetType(IloIntIndex) { return IntIndex; }
  IloInt makeValue(IloModelEvaluatorI*) const;
};

inline ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, const IloOplIntExpr & ext) {
  ext.display(out);
  return (out);
}


class IloNumCollectedMapI;
class IloNumCollectedMapIteratorI : public IloMemoryManagerObjectI {
protected:
  IloNumCollectedMapI* _collectedMap;
  virtual IloNumVar getSubstitutionAsNumVar() const = 0;
  virtual IloIntVar getSubstitutionAsIntVar() const = 0;
  virtual IloNum getSubstitutionAsNum() const = 0;
  virtual IloInt getSubstitutionAsInt() const = 0;
public:
  IloNumCollectedMapIteratorI(IloNumCollectedMapI* collectedMap);
  virtual ~IloNumCollectedMapIteratorI();
  virtual IloInt getPos() const = 0;
  virtual IloInt getIndex() const = 0;
  IloNum getNum() const;
  IloNumVar getNumVar() const;
  IloInt getInt() const;
  IloIntVar getIntVar() const;
  virtual IloBool ok() const = 0;
  virtual void operator++() = 0;
  virtual void end() = 0;
};

class IloNumCollectedMap : public IloExtractable {
  ILOEXTRHANDLEDECL(IloNumCollectedMap, IloExtractable)
public:
  enum Type {
    NumConst,
    IntConst,
    NumVar,
    IntVar};
private:
    void init(IloEnv env, IloGeneratorArray gs, IloNumExprArg expr, Type t, const char* name);

public:
  IloNumCollectedMap(IloEnv env, IloGeneratorArray gs, IloNumExprArg expr, Type t=NumVar, const char* name=0) :
    IloExtractable(0) {
       IloAssert(env.getImpl(), "IloNumCollectedMap::IloNumCollectedMap : Using empty IloEnv handle");
       init(env.getImpl(), gs, expr, t, name);
    }

  IloNumCollectedMap(IloEnv env, IloGeneratorArray gs, IloIntExprArg expr, Type t=IntVar, const char* name=0) :
    IloExtractable(0) {
       IloAssert(env.getImpl(), "IloNumCollectedMap::IloNumCollectedMap : Using empty IloEnv handle");
       init(env.getImpl(), gs, expr, t, name);
    }

  Type getType() const;
  class Iterator {
    IloNumCollectedMapIteratorI* _impl;
  public:
    Iterator(IloNumCollectedMap CollectedMap, IloModelEvaluator eval);
    IloInt getPos() const {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      return _impl->getPos();
    }
    IloInt getIndex() const {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      return _impl->getIndex();
    }
    IloNumVar getNumVar() const {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      return _impl->getNumVar();
    }
    IloNum getNum() const {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      return _impl->getNum();
    }
    IloIntVar getIntVar() const {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      return _impl->getIntVar();
    }
    IloInt getInt() const {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      return _impl->getInt();
    }
    IloBool ok() const {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      return _impl->ok();
    }
    void operator++() {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      ++(*_impl);
    }
    void end() {
      IloAssert(_impl, "Using empty IloNumCollectedMap::Iterator handle");
      _impl->end();
      delete _impl;
    }
  };
};

class IloNumCollectedMapI : public IloExtractableI {
  ILOEXTRDECL
  ILOEXTRCONSTRUCTOR0DECL(IloNumCollectedMapI, IloExtractableI)
public:
  virtual IloInt getNbDim() const = 0;
  virtual IloBool isIntIndexed() const = 0;
  virtual IloIntRangeExprI* getRange() const = 0;
  virtual IloInt getMinIndex(IloModelEvaluatorI*) const = 0;
  virtual IloNumCollectedMap::Type getType() const = 0;
  virtual IloNumCollectedMapIteratorI* getIterator(IloModelEvaluatorI*) const = 0;
  virtual void lockExprs() = 0;
  virtual void releaseExprs() = 0;
};


inline
IloNumCollectedMap::Iterator::Iterator(IloNumCollectedMap collectedMap, IloModelEvaluator eval) : _impl(0) {
  IloAssert(collectedMap.getImpl(), "Using empty IloNumCollectedMap handle");
  IloAssert(eval.getImpl(), "Using empty IloModelEvaluator handle");
 _impl = collectedMap.getImpl()->getIterator(eval.getImpl());
}

inline
IloNumCollectedMap::Type IloNumCollectedMap::getType() const {
  IloAssert(getImpl(), "Using empty handle");
  return getImpl()->getType();
}

inline
IloNumVar IloNumCollectedMapIteratorI::getNumVar() const {
  IloAssert(_collectedMap->getType()==IloNumCollectedMap::NumVar, "IloNumCollectedMap::Iterator::getNumVar : Bad type of item.");
  return getSubstitutionAsNumVar();
}
inline
IloIntVar IloNumCollectedMapIteratorI::getIntVar() const {
  IloAssert(_collectedMap->getType()==IloNumCollectedMap::IntVar, "IloNumCollectedMap::Iterator::getIntVar : Bad type of item.");
  return getSubstitutionAsIntVar();
}

inline
IloNum IloNumCollectedMapIteratorI::getNum() const {
  IloAssert(_collectedMap->getType()==IloNumCollectedMap::NumConst, "IloNumCollectedMap::Iterator::getNum : Bad type of item.");
  return getSubstitutionAsNum();
}
inline
IloInt IloNumCollectedMapIteratorI::getInt() const {
  IloAssert(_collectedMap->getType()==IloNumCollectedMap::IntConst, "IloNumCollectedMap::Iterator::getInt : Bad type of item.");
  return getSubstitutionAsInt();
}

//---------- Exceptions
class IloNumCollectedMapException  : public IloException {
  const IloNumCollectedMapI* _map;
  const IloExtractableI* _obj;
public:
  IloNumCollectedMapException (const IloNumCollectedMapI*, const IloExtractableI*);
  virtual void print(ILOSTD(ostream)& out) const;
  const IloNumCollectedMapI* getCollectedMap() const { return (_map); }
  const IloExtractableI* getExtractable() const { return (_obj); }
  void setContext(IloExtractableI* context) { _obj = context; }
};
class IloNumCollectedMapOutOfBoundException  : public IloNumCollectedMapException {
  const IloMapItem _index;
public:
  IloNumCollectedMapOutOfBoundException (const IloNumCollectedMapI*, const IloMapItem, const IloExtractableI*);
  void print(ILOSTD(ostream)& out) const;
  const IloMapItem getMapItem() const { return (_index); }
};
class IloWrongNumCollectedMapDimensionException  : public IloNumCollectedMapException {
  const IloInt _expectedNbDim;
public:
  IloWrongNumCollectedMapDimensionException (const IloNumCollectedMapI*, const IloInt, const IloExtractableI*);
  const IloInt getExpectedNbOfDimension() const { return (_expectedNbDim); }
  void print(ILOSTD(ostream)& out) const;
};

//---------- IloRealNumCollectedMap

class IloRealNumCollectedMapI : public IloNumCollectedMapI {
  ILOEXTRMEMBERS4DECL(IloRealNumCollectedMapI, IloNumCollectedMapI,
         IloComprehensionI*, _comp,
         IloIntRangeExprI*, _range,
         IloNumExprI*, _expr,
         IloNumCollectedMap::Type, _type)
public:
  ~IloRealNumCollectedMapI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloComprehensionI* getComprehension() const {return _comp;}
  IloNumExprI* getExpr() const {return _expr;}
  virtual IloInt getNbDim() const;
  virtual IloBool isIntIndexed() const;
  virtual IloIntRangeExprI* getRange() const {return _range;}
  virtual IloInt getMinIndex(IloModelEvaluatorI*) const;
  virtual IloNumCollectedMap::Type getType() const {return  _type;}
  virtual IloNumCollectedMapIteratorI* getIterator(IloModelEvaluatorI*) const;
  void lockExprs();
  void releaseExprs();
};

class IloRealNumCollectedMapIteratorI : public IloNumCollectedMapIteratorI {
  IloModelEvaluatorI* _eval;
  IloModelEvaluator::Iterator _subIt;
  IloInt _pos;
  IloNumVar getSubstitutionAsNumVar() const;
  IloIntVar getSubstitutionAsIntVar() const;
  IloNum getSubstitutionAsNum() const;
  IloInt getSubstitutionAsInt() const;
public:
  IloRealNumCollectedMapIteratorI(IloRealNumCollectedMapI* CollectedMap, IloModelEvaluatorI* eval);
  virtual IloInt getPos() const;
  virtual IloInt getIndex() const;
  virtual IloBool ok() const;
  virtual void operator++();
  virtual void end();
};

//---------- IloMapAsNumCollectedMap


class IloMapAsNumCollectedMapI : public IloNumCollectedMapI {
  ILOEXTRMEMBERS2DECL(IloMapAsNumCollectedMapI, IloNumCollectedMapI,
         IloMapI*, _map,
         IloNumCollectedMap::Type, _type)
public:
  ~IloMapAsNumCollectedMapI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloMapI* getMap() const {return _map;}
  virtual IloInt getNbDim() const;
  virtual IloBool isIntIndexed() const;
  virtual IloIntRangeExprI* getRange() const {return 0;}
  virtual IloInt getMinIndex(IloModelEvaluatorI*) const;
  virtual IloNumCollectedMap::Type getType() const {return  _type;}
  virtual IloNumCollectedMapIteratorI* getIterator(IloModelEvaluatorI*) const;
  void lockExprs();
  void releaseExprs();
};

class IloMapAsNumCollectedMapIteratorI : public IloNumCollectedMapIteratorI {
  IloModelEvaluatorI* _eval;
  IloMapIterator _subIt;
  IloInt _pos;
  IloNumVar getSubstitutionAsNumVar() const;
  IloIntVar getSubstitutionAsIntVar() const;
  IloNum getSubstitutionAsNum() const;
  IloInt getSubstitutionAsInt() const;
public:
  IloMapAsNumCollectedMapIteratorI(IloMapAsNumCollectedMapI* CollectedMap, IloModelEvaluatorI* eval);
  virtual IloInt getPos() const;
  virtual IloInt getIndex() const;
  virtual IloBool ok() const;
  virtual void operator++();
  virtual void end();
};


//---------- IloAppendedNumCollectedMap


class IloAppendedNumCollectedMapI : public IloNumCollectedMapI {
  ILOEXTRMEMBERS2DECL(IloAppendedNumCollectedMapI, IloNumCollectedMapI,
         IloNumCollectedMapI*, _map1,
         IloNumCollectedMapI*, _map2)
public:
  ~IloAppendedNumCollectedMapI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloNumCollectedMapI* getMap1() const {return _map1;}
  IloNumCollectedMapI* getMap2() const {return _map2;}
  virtual IloInt getNbDim() const;
  virtual IloBool isIntIndexed() const;
  virtual IloIntRangeExprI* getRange() const {return 0;}
  virtual IloInt getMinIndex(IloModelEvaluatorI*) const;
  virtual IloNumCollectedMap::Type getType() const {return  _map1->getType();}
  virtual IloNumCollectedMapIteratorI* getIterator(IloModelEvaluatorI*) const;
  void lockExprs();
  void releaseExprs();
};

class IloAppendedNumCollectedMapIteratorI : public IloNumCollectedMapIteratorI {
  IloModelEvaluatorI* _eval;
  IloNumCollectedMap::Iterator _subIt;
  IloInt _which;
  IloInt _pos;
  IloNumVar getSubstitutionAsNumVar() const;
  IloIntVar getSubstitutionAsIntVar() const;
  IloNum getSubstitutionAsNum() const;
  IloInt getSubstitutionAsInt() const;
public:
  IloAppendedNumCollectedMapIteratorI(IloAppendedNumCollectedMapI* CollectedMap, IloModelEvaluatorI* eval);
  virtual IloInt getPos() const;
  virtual IloInt getIndex() const;
  virtual IloBool ok() const;
  virtual void operator++();
  virtual void end();
};


///////////////////////////
// Map CP Constraints
class IloAllDiffCollectedMapI : public IloConstraintI {
  ILOEXTRMEMBERS1DECL(IloAllDiffCollectedMapI, IloExtractableI,
         IloNumCollectedMapI*, _collectedMap)
public:
  ~IloAllDiffCollectedMapI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloNumCollectedMapI* getCollectedMap() const {return _collectedMap;}
};

class IloAllMinDistanceCollectedMapI : public IloConstraintI {
  ILOEXTRDECL
  IloNumCollectedMapI* _collectedMap;
  IloOplIntExpr _offset;
public:
    IloAllMinDistanceCollectedMapI(IloEnvI* env,
				   IloNumCollectedMapI* collectedMap,
				   IloOplIntExpr offset,
				   const char* name = 0);
  virtual ~IloAllMinDistanceCollectedMapI();
  ILOEXTROTHERDECL
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloNumCollectedMapI* getCollectedMap() const {return _collectedMap;}
  IloOplIntExpr getOffset() const {return _offset;}
};

class IloInverseCollectedMapI : public IloConstraintI {
  ILOEXTRMEMBERS2DECL(IloInverseCollectedMapI, IloExtractableI,
         IloNumCollectedMapI*, _collectedMap1,
         IloNumCollectedMapI*, _collectedMap2)
public:
  ~IloInverseCollectedMapI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloNumCollectedMapI* getCollectedMap1() const {return _collectedMap1;}
  IloNumCollectedMapI* getCollectedMap2() const {return _collectedMap2;}
};

class IloPackCollectedMapI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloNumCollectedMapI* _load;
  IloNumCollectedMapI* _where;
  IloNumCollectedMapI* _weight;  // csts
  IloOplIntExpr _numBinUsed;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloPackCollectedMapI(IloEnvI* env, IloNumCollectedMapI* ld, IloNumCollectedMapI* wh, IloNumCollectedMapI* we,
	      const char* name = 0);
  IloPackCollectedMapI(IloEnvI* env, IloNumCollectedMapI* ld, IloNumCollectedMapI* wh, IloNumCollectedMapI* we,
	      IloOplIntExpr numBinUsed, const char* name = 0);
  virtual ~IloPackCollectedMapI();
  IloNumCollectedMapI* getLoadMap() const {return _load;}
  IloNumCollectedMapI* getWhereMap() const {return _where;}
  IloNumCollectedMapI* getWeightMap() const {return _weight;}
  IloOplIntExpr getNumBinUsed() const {return _numBinUsed;}
  IloBool withNumBinUsed() const {return _numBinUsed.isValid();}
  ILOEXTROTHERDECL
};

class IloTableConstraintMapI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloIntExprArray  _exprs;
  IloTupleSetExprI*  _values;
  IloBool        _compatible;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTableConstraintMapI(IloEnvI* env, IloIntExprArray m,
			 IloTupleSetExprArg vals, IloBool compat, const char* name = 0);
  virtual ~IloTableConstraintMapI();
  IloIntExprArray getExprArray() const {return _exprs;}
  IloTupleSetExprI* getValues() const {return _values;}
  IloBool isCompatible() const {return _compatible; }
  ILOEXTROTHERDECL
};

class IloCountCollectedMapI : public IloIntExprI {
  ILOEXTRDECL
private:
  IloNumCollectedMapI* _collectedMap;
  IloOplIntExpr _value; // IloInt or IloIntIndex
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloCountCollectedMapI(IloEnvI* env,
	       IloNumCollectedMapI* collectedMap, IloOplIntExpr count,
	       const char* name=0);
  virtual ~IloCountCollectedMapI();
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumCollectedMapI* getCollectedMap() const {return _collectedMap;}
  IloOplIntExpr getValue() const {return _value;}
  ILOEXTROTHERDECL
};


class IloElementCollectedMapIntExprI : public IloIntExprI {
  ILOEXTRDECL
private:
  IloNumCollectedMapI* _collectedMap;
  IloOplIntExpr _index; // IloInt or IloIntIndex
  IloBool _displayAsSlot;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloElementCollectedMapIntExprI(IloEnvI* env,
	       IloNumCollectedMapI* collectedMap, IloOplIntExpr index,
	       IloBool asSlot =IloFalse, const char* name=0);
  virtual ~IloElementCollectedMapIntExprI();
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumCollectedMapI* getCollectedMap() const {return _collectedMap;}
  IloOplIntExpr getIndex() const {return _index;}
  IloBool displayAsSlot() const {return _displayAsSlot;}
  ILOEXTROTHERDECL
};

class IloElementCollectedMapNumExprI : public IloNumExprI {
  ILOEXTRDECL
private:
  IloNumCollectedMapI* _collectedMap;
  IloOplIntExpr _index; // IloInt or IloIntIndex
  IloBool _displayAsSlot;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloElementCollectedMapNumExprI(IloEnvI* env,
	       IloNumCollectedMapI* collectedMap, IloOplIntExpr index,
	       IloBool asSlot =IloFalse, const char* name=0);
  virtual ~IloElementCollectedMapNumExprI();
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumCollectedMapI* getCollectedMap() const {return _collectedMap;}
  IloOplIntExpr getIndex() const {return _index;}
  IloBool displayAsSlot() const {return _displayAsSlot;}
  ILOEXTROTHERDECL
};

class IloLexCollectedMapI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloNumCollectedMapI*  _collectedMap1;
  IloNumCollectedMapI*  _collectedMap2;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloLexCollectedMapI(IloEnvI* env,
	     IloNumCollectedMapI* m1, IloNumCollectedMapI* m2, const char* name = 0);
  virtual ~IloLexCollectedMapI();
  IloNumCollectedMapI* getCollectedMap1() const {return _collectedMap1;}
  IloNumCollectedMapI* getCollectedMap2() const {return _collectedMap2;}
  ILOEXTROTHERDECL
};

class IloStandardDeviationCollectedMapI : public IloNumExprI {
  ILOEXTRDECL
private:
  IloNumCollectedMapI* _collectedMap;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloStandardDeviationCollectedMapI(IloEnvI* env,
			   IloNumCollectedMapI* map, const char* name=0);
  virtual ~IloStandardDeviationCollectedMapI();
  virtual IloNum eval(const IloAlgorithm alg) const;
  IloNumCollectedMapI* getCollectedMap() const {return _collectedMap;}
  ILOEXTROTHERDECL
};


#endif

