// --------------------------------------------------*- C++ -*---------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilorttiH
#define __CONCERT_ilorttiH

class ILO_EXPORTED IloRtti {
public:
  typedef IloInt TypeIndex;
  typedef void (*TypeInfo)();
private:
  static TypeIndex _counter;
protected:
  static TypeIndex getNewRtti() {
    return ++_counter;
  }
public:
  virtual ~IloRtti();

  static  TypeIndex    GetTypeIndex();
  virtual TypeIndex    getTypeIndex() const;

  static  TypeInfo    GetTypeInfo();
  static  TypeInfo    GetBaseTypeInfo();
  virtual TypeInfo    getTypeInfo() const;
  virtual IloBool     isType (TypeInfo typeinfo) const;
  virtual const char* getRttiTypeName() const = 0;
  virtual IloInt getExtractableSize() const = 0;
};

typedef IloRtti::TypeIndex IloTypeIndex;
typedef IloRtti::TypeInfo IloTypeInfo;

IloTypeInfo IloGetBaseTypeInfo(IloTypeInfo);

#ifdef USE_MEMORY_VALIDATOR
#define ILOMEMORYVALIDATORDECL											\
  void* operator new (size_t sz, const IloEnv& env);					\
  void* operator new (size_t sz, const IloEnvI* env);					\
  void* operator new (size_t s, IloAllocationStack* heap);				\
  void* operator new (size_t sz, const IloGenAlloc * gen);				
#else
#define ILOMEMORYVALIDATORDECL
#endif

# define ILORTTIDECL                                                    \
  static IloTypeIndex _rttiIndex;                                       \
public:                                                                 \
  static  IloTypeInfo GetTypeInfo();                                    \
  static  IloTypeInfo GetBaseTypeInfo();                                \
  virtual IloTypeInfo getTypeInfo() const;                              \
  virtual IloTypeInfo getBaseTypeInfo() const;                          \
  virtual IloBool     isType(IloTypeInfo typeinfo) const;               \
  virtual IloTypeIndex getTypeIndex() const;                            \
  virtual IloTypeIndex getBaseTypeIndex() const;                        \
  static  IloTypeIndex GetTypeIndex();                                  \
  static  void        InitTypeIndex();                                  \
  virtual const char* getRttiTypeName() const;                          \
  static  const char* GetRttiTypeName();                                \
  virtual IloInt      getExtractableSize() const;                       \
  static  IloInt      GetExtractableSize();								\
  ILOMEMORYVALIDATORDECL

// The ILODUMMYRTTI macro is important as it prevent the msvc compiler to merge
// the GetXXXTypeInfo functions into a single one

#ifdef ILO_MSVC
# define ILODUMMYRTTI(_this)\
  if (_rttiIndex == _rttiIndex - 1)\
     ILOSTD(cout) << (_this);
#else
# define ILODUMMYRTTI(_this)
#endif

//  return (IloRtti::TypeInfo(&_this::GetBaseTypeInfo));

#ifndef USE_OLD_RTTI_MACROS
#define ILONOTEMPLATE()
#define ILOGENTEMPLATE_1(a)	template < a >
#define ILOGENTEMPLATE_2(a,b)	template < a , b >
#define ILOGENTEMPLATE_3(a,b,c) template < a , b , c >

#ifdef USE_MEMORY_VALIDATOR
#define MV_REGISTER_TYPE	mvConcertRegisterType( _rttiIndex, GetRttiTypeName() );
#else
#define MV_REGISTER_TYPE
#endif

#define ILORTTIN(_thisMacro, _this, _baseMacro, _base, _templateMacro, _template) \
_templateMacro _template \
IloTypeInfo _thisMacro _this::GetBaseTypeInfo() { \
  ILODUMMYRTTI(ILO_STRINGIZE(_thisMacro _this))\
  const IloRtti::TypeInfo type = _baseMacro _base::GetTypeInfo(); \
  return type; \
} \
_templateMacro _template \
IloTypeInfo _thisMacro _this::GetTypeInfo() { \
  ILODUMMYRTTI(ILO_STRINGIZE(_thisMacro _this))\
  return (IloRtti::TypeInfo(&_thisMacro _this::GetBaseTypeInfo));\
} \
_templateMacro _template \
IloTypeInfo _thisMacro _this::getTypeInfo() const { \
  return (IloRtti::TypeInfo(&_thisMacro _this::GetBaseTypeInfo)); \
} \
_templateMacro _template \
IloTypeInfo _thisMacro _this::getBaseTypeInfo() const { \
  const IloRtti::TypeInfo type = _baseMacro _base::GetTypeInfo(); \
  return type; \
} \
_templateMacro _template \
IloBool _thisMacro _this::isType (IloTypeInfo typeinfo) const { \
  return (typeinfo == GetTypeInfo()  || \
          _baseMacro _base::isType (typeinfo)  ); \
} \
_templateMacro _template \
IloTypeIndex _thisMacro _this::_rttiIndex = 0; \
_templateMacro _template \
IloTypeIndex _thisMacro _this::getTypeIndex() const { \
  return _rttiIndex; \
} \
_templateMacro _template \
IloTypeIndex _thisMacro _this::getBaseTypeIndex() const { \
  return _baseMacro _base::GetTypeIndex(); \
} \
_templateMacro _template \
void _thisMacro _this::InitTypeIndex() { \
  if (_rttiIndex == 0) { \
    IloEnvI::Lock(); \
	if (_rttiIndex == 0) { \
		_rttiIndex = IloRtti::getNewRtti(); \
		MV_REGISTER_TYPE \
	} \
    IloEnvI::Unlock(); \
  } \
} \
_templateMacro _template \
IloTypeIndex _thisMacro _this::GetTypeIndex() { \
  InitTypeIndex(); \
  return _rttiIndex; \
} \
_templateMacro _template \
const char* _thisMacro _this::getRttiTypeName() const { \
  return name2(_thisMacro,_STRINGIZE) _this; \
} \
_templateMacro _template \
const char* _thisMacro _this::GetRttiTypeName() { \
  return name2(_thisMacro,_STRINGIZE) _this; \
}\
_templateMacro _template \
IloInt _thisMacro _this::GetExtractableSize() { \
  return sizeof(_thisMacro _this); \
}\
_templateMacro _template \
IloInt _thisMacro _this::getExtractableSize() const {\
  return sizeof(_thisMacro _this); \
}

#define ILOEMPTYMACROARGUMENT
#define ILOSIMPLECLASS(X)	X
#define ILOTEMPLATECLASS_1(X,Y) X < Y >
#define ILOTEMPLATECLASS_2(X,Y,Z) X < Y , Z >
#define ILOTEMPLATECLASS_3(X,Y,Z,K) X < Y, Z, K >

#define ILOSIMPLECLASS_STRINGIZE(X)	#X
#define ILOTEMPLATECLASS_1_STRINGIZE(X,Y) #X "<" #Y ">"
#define ILOTEMPLATECLASS_2_STRINGIZE(X,Y,Z) #X "<" #Y "," #Z ">"
#define ILOTEMPLATECLASS_3_STRINGIZE(X,Y,Z,K) #X "<" #Y "," #Z "," #K ">"

#define ILORTTI1(_this, _base, _t1)  \
ILORTTIN(ILOSIMPLECLASS,(_this),ILOSIMPLECLASS,(_base),ILOGENTEMPLATE_1,(_t1))

#define ILORTTI2(_this,_base,_t1,_t2) \
ILORTTIN(ILOSIMPLECLASS,(_this),ILOSIMPLECLASS,(_base),ILOGENTEMPLATE_2,(_t1,_t2))

#define ILORTTI3(_this,_base,_t1,_t2,t3) \
ILORTTIN(ILOSIMPLECLASS,(_this),ILOSIMPLECLASS,(_base),ILOGENTEMPLATE_3,(_t1,_t2,t3))


#ifdef USE_MEMORY_VALIDATOR

void mvConcertCustomAlloc( void* ptr, const size_t sz, const int typeId );
void mvConcertRegisterType( const int typeId, const char* typeName );

#define ILOMEMORYVALIDATOR(_this)															\
	void* _this::operator new (size_t sz, const IloEnv& env){                               \
		return _this::operator new (sz, env.getImpl());                                     \
	}                                                                                       \
	void* _this::operator new (size_t sz, const IloEnvI* env){								\
		IloAny ret = env->alloc(sz);                                                        \
		mvConcertCustomAlloc( ret, sz, _this::GetTypeIndex() );								\
		return ret;																			\
	}																						\
	void* _this::operator new (size_t sz, IloAllocationStack* l){							\
		if (l) {																			\
			IloAny ret = l->allocate((unsigned int) sz);									\
			mvConcertCustomAlloc( ret, sz, _this::GetTypeIndex() );							\
			return ret;																		\
		} else {																			\
			void* tmp;																		\
			ILOPROTECTNEW(tmp, new char[sz]);												\
			mvConcertCustomAlloc( tmp, sz, _this::GetTypeIndex() );							\
			return tmp;																		\
		}																					\
	}																						\
	void* _this::operator new (size_t sz, const IloGenAlloc* env){							\
		IloAny ret = env->alloc(sz);														\
		mvConcertCustomAlloc( ret, sz, _this::GetTypeIndex() );								\
		return ret;																			\
	}																						
#else
#	define ILOMEMORYVALIDATOR(_this)
#endif

#define ILORTTI( _this, _base )													\
ILOMEMORYVALIDATOR( _this )														\
ILORTTIN(ILOSIMPLECLASS,( _this ),ILOSIMPLECLASS,( _base ),ILONOTEMPLATE,())
#else
#define ILOEMPTYMACROARGUMENT
#define ILORTTI(_base,_this)			\
 ILORTTI1(_base,				\
 	 _this,					\
 	 ILOEMPTYMACROARGUMENT)

#define ILORTTI1(_this, _base,_template) \
_template \
IloTypeInfo _this::GetBaseTypeInfo() { \
  ILODUMMYRTTI(#_this)\
  const IloRtti::TypeInfo type = _base::GetTypeInfo(); \
  return type; \
} \
_template \
IloTypeInfo _this::GetTypeInfo() { \
  ILODUMMYRTTI(#_this)\
  return (IloRtti::TypeInfo((void*)&_this::GetBaseTypeInfo));\
} \
_template \
IloTypeInfo _this::getTypeInfo() const { \
  return (IloRtti::TypeInfo((void*)&_this::GetBaseTypeInfo)); \
} \
_template \
IloTypeInfo _this::getBaseTypeInfo() const { \
  const IloRtti::TypeInfo type = _base::GetTypeInfo(); \
  return type; \
} \
_template \
IloBool _this::isType (IloTypeInfo typeinfo) const { \
  return (typeinfo == GetTypeInfo()  || \
          _base::isType (typeinfo)  ); \
} \
_template \
IloTypeIndex _this::_rttiIndex = 0; \
_template \
IloTypeIndex _this::getTypeIndex() const { \
  return _rttiIndex; \
} \
_template \
IloTypeIndex _this::getBaseTypeIndex() const { \
  return _base::GetTypeIndex(); \
} \
_template \
void _this::InitTypeIndex() { \
  if (_rttiIndex == 0) { \
    IloEnvI::Lock(); \
    if (_rttiIndex == 0) \
		_rttiIndex = IloRtti::getNewRtti(); \
    IloEnvI::Unlock(); \
  } \
} \
_template \
IloTypeIndex _this::GetTypeIndex() { \
  InitTypeIndex(); \
  return _rttiIndex; \
} \
_template \
const char* _this::getRttiTypeName() const { \
  return #_this; \
} \
_template \
const char* _this::GetRttiTypeName() { \
  return #_this; \
}\
_template \
IloInt _this::GetExtractableSize() { \
  return sizeof(_this); \
}\
_template \
IloInt _this::getExtractableSize() const {\
  return sizeof(_this); \
}

#endif

#endif
