// -------------------------------------------------------*- C++ -*----------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilosymbolH
#define __CONCERT_ilosymbolH

#ifndef __CONCERT_iloanyH
#include <ilconcert/iloany.h>
#endif

#ifndef __CONCERT_iloanyexpriH
#include <ilconcert/iloanyexpri.h>
#endif

#ifndef __CONCERT_iloadvcollectionH
#include <ilconcert/iloadvcollection.h>
#endif

class IloAnyCollection;
class IloSymbolSet;

typedef IloDefaultArrayI IloSymbolArrayI;
// --------------------------------------------------------------
// Symbols
// --------------------------------------------------------------
class IloSymbolI : public IloRttiEnvObjectI {
  ILORTTIDECL
  friend class IloSymbolTable;
private:
  char* _str;
  IloSymbolI(IloEnvI* env, const char* str);
public:
  ~IloSymbolI();
  const char* getString() const {return _str;}
  void display(ILOSTD(ostream)& out) const;
};

//-----------------------------------------------


class ILO_EXPORTED IloSymbol {
private:
  IloSymbolI* _impl;
public:
  IloSymbol():_impl(0) {}
  IloSymbol(IloSymbolI* impl):_impl(impl) {}

  IloSymbolI* getImpl() const { return _impl; }

  const char* getString() const {
    IloAssert (getImpl(), "accessing IloSymbol through 0 handle");
    return (_impl->getString());
  }
};

inline ILOSTD(ostream)& operator<< (ILOSTD(ostream)& str, const IloSymbol& s) {
  IloAssert (s.getImpl(), "accessing IloSymbol through 0 handle");
  s.getImpl()->display(str);
  return str;
}


// --------------------------------------------------------------
// Symbol Expressions
// --------------------------------------------------------------

#define IloSymbolExprArgI IloAnyExprI

//-----------------------------------------------------------------
class ILO_EXPORTED IloSymbolExprArg : public IloExtractable {
  ILOEXTRHANDLE(IloSymbolExprArg, IloExtractable)
};


class IloSymbolEncoderI : public IloEncoderI {
public:
  IloSymbolEncoderI():IloEncoderI() {}
  virtual ~IloSymbolEncoderI(){}
  virtual IloBool isSymbolEncoder() const;
  virtual IloInt anyToInt(IloAny e) const;
  virtual IloAny intToAny(IloInt e) const;
  virtual void display(ILOSTD(ostream)& out, IloAny e) const;
};

// --------------------------------------------------------------
// Symbol Constraints
// --------------------------------------------------------------

//-----------------------------------------------------------------
IloConstraint operator==(const IloSymbolExprArg expr1,
			 const IloSymbolExprArg expr2);

IloConstraint operator==(const IloSymbolExprArg expr, const char* value);

inline IloConstraint operator==(const char* val, const IloSymbolExprArg expr) {
  return (expr == val);
}

IloConstraint operator==(const IloSymbolExprArg expr, IloSymbol value);

inline IloConstraint operator==(IloSymbol val, const IloSymbolExprArg expr) {
  return (expr == val);
}

IloConstraint operator!=(const IloSymbolExprArg expr1, const IloSymbolExprArg expr2);

IloConstraint operator!=(const IloSymbolExprArg expr, const char* value);

inline IloConstraint operator!=(const char* val, const IloSymbolExprArg expr) {
  return (expr != val);
}

IloConstraint operator!=(const IloSymbolExprArg expr, IloSymbol value);

inline IloConstraint operator!=(IloSymbol val, const IloSymbolExprArg expr) {
  return (expr != val);
}


//

IloConstraint operator<(const IloSymbolExprArg expr1, const IloSymbolExprArg expr2);

IloConstraint operator<(const IloSymbolExprArg expr, const char* value);

inline IloConstraint operator<(const char* val, const IloSymbolExprArg expr) {
  return (expr < val);
}

IloConstraint operator<(const IloSymbolExprArg expr, IloSymbol value);

inline IloConstraint operator<(IloSymbol val, const IloSymbolExprArg expr) {
  return (expr < val);
}

IloConstraint operator>(const IloSymbolExprArg expr1, const IloSymbolExprArg expr2);

IloConstraint operator>(const IloSymbolExprArg expr, const char* value);

inline IloConstraint operator>(const char* val, const IloSymbolExprArg expr) {
  return (expr > val);
}

IloConstraint operator>(const IloSymbolExprArg expr, IloSymbol value);

inline IloConstraint operator>(IloSymbol val, const IloSymbolExprArg expr) {
  return (expr > val);
}

IloConstraint operator<=(const IloSymbolExprArg expr1, const IloSymbolExprArg expr2);

IloConstraint operator<=(const IloSymbolExprArg expr, const char* value);

inline IloConstraint operator<=(const char* val, const IloSymbolExprArg expr) {
  return (expr <= val);
}

IloConstraint operator<=(const IloSymbolExprArg expr, IloSymbol value);

inline IloConstraint operator<=(IloSymbol val, const IloSymbolExprArg expr) {
  return (expr <= val);
}


IloConstraint operator>=(const IloSymbolExprArg expr1, const IloSymbolExprArg expr2);

IloConstraint operator>=(const IloSymbolExprArg expr, const char* value);

inline IloConstraint operator>=(const char* val, const IloSymbolExprArg expr) {
  return (expr >= val);
}

IloConstraint operator>=(const IloSymbolExprArg expr, IloSymbol value);

inline IloConstraint operator>=(IloSymbol val, const IloSymbolExprArg expr) {
  return (expr >= val);
}


//--------------------------------------------------------------------
// Symbol arrays
//--------------------------------------------------------------------
typedef IloArray<IloSymbol> IloSymbolArrayBase;

class ILO_EXPORTED IloSymbolArray : public IloSymbolArrayBase {
public:
  typedef IloDefaultArrayI ImplClass;
  IloSymbolArray(IloDefaultArrayI* i=0) : IloSymbolArrayBase(i) {}
  IloSymbolArray(const IloSymbolArray& copy) : IloSymbolArrayBase(copy) {}
  IloSymbolArray(const IloEnv env, IloInt n = 0) : IloSymbolArrayBase(env, n) {}
  IloSymbolArray(const IloMemoryManager env, IloInt n = 0) : IloSymbolArrayBase(env, n) {}

  IloSymbolArray(const IloEnv env, IloInt n, const IloSymbol v0);
  IloSymbolArray(const IloEnv env, IloAnyArray from);
  IloAnyArray toAnyArray() const;

  using IloArray<IloSymbol>::add;
  void add(const char* name);
  IloSymbolArray(const IloEnv env, IloInt n, const char* v0);
  IloBool contains(const IloSymbol symbol) const;
  IloBool contains(const char* name) const;
  const IloSymbol& operator[] (IloInt i) const {
    return IloSymbolArrayBase::operator[](i);
  }
  IloSymbol& operator[] (IloInt i) {
    return IloSymbolArrayBase::operator[](i);
  }
  IloSymbolExprArg operator[] (IloIntExprArg intExp) const;
};

#define IloSymbolExprI IloAnyExprI


class IloSymbolArrayElementI : public IloSymbolExprI {
  ILOEXTRDECL
private:
  IloIntExprI* _index;
  IloSymbolArray  _array;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloSymbolArrayElementI(IloEnvI* env, IloIntExprI* index, IloSymbolArray array):
    IloSymbolExprI(env), _index(index->intLockExpr()), _array(array) {}
  virtual ~IloSymbolArrayElementI();
  void display(ILOSTD(ostream)& out) const;
  IloSymbolArray getArray() const { return _array; }
  IloIntExprI* getIndex() const { return _index; }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloEncoderI* getEncoder() const {
    return getEnv()->getSymbolEncoder();
  }
};


typedef IloHandleSet<IloSymbol , IloSymbolI > IloSymbolSetBase;
class IloSymbolSet: public IloSymbolSetBase{
public:
	class Iterator : public IloAnyDefaultDataIterator {
		public:
			Iterator() : IloAnyDefaultDataIterator() {}

			Iterator(const IloSymbolSet coll): IloAnyDefaultDataIterator(coll.getImpl()->getEnv()->getGeneralAllocator(),
				coll.getImpl()) {
					reset();
				}
			virtual ~Iterator(){}

			const char* operator*() {
					return IloSymbol((IloSymbolI*)IloAnyDefaultDataIterator::operator*()).getImpl()->getString();
				}

#ifdef CPPREF_GENERATION
      IloBool ok() const {
       	return _ok;
      }

      void operator++() {
        _ok = next();
      }
#endif
		};

public:
	IloSymbolSet(const IloEnv env,	const IloSymbolArray array, IloBool withIndex=IloFalse);

	IloSymbolSet(const IloEnv env,	const IloAnyArray array, IloBool withIndex=IloFalse);
	IloSymbolSet(const IloEnv env,	const IloSymbolSet set);
	IloSymbolSet(const IloEnv env, IloBool withIndex=IloFalse);
	IloSymbolSet(IloAnySetI* impl=0) : IloSymbolSetBase(impl) { if (impl) impl->setType(SymbolSet);}
	IloSymbolSet(IloSymbolExprArg e, IloGenerator g);
	IloSymbolSet(IloSymbolExprArg e, IloGeneratorArray generators);

	using IloSymbolSetBase::add;
	using IloSymbolSetBase::remove;
	using IloSymbolSetBase::contains;
	using IloSymbolSetBase::setIntersection;

	void add(const char* elt);

	void remove(const char* elt);

	void setIntersection(const char* elt);

	IloBool contains(const char* elt) const;

	IloBool isSymbolSet() const { return IloTrue; }

	using IloSymbolSetBase::getNext;

	IloSymbol getNext(const char* value, IloInt n=1) const{
		IloAssert(getImpl() != 0, "IloSet: Using empty handle");
		IloSymbol s = getImpl()->getEnv()->makeSymbol(value);
		return IloSymbol((IloSymbolI*)((IloAnySetI*)_impl)->getNext(s.getImpl(),n));
	}
	using IloSymbolSetBase::getPrevious;

	IloSymbol getPrevious(const char* value, IloInt n=1) const{
		IloAssert(getImpl() != 0, "IloSet: Using empty handle");
		IloSymbol s = getImpl()->getEnv()->makeSymbol(value);
		return IloSymbol((IloSymbolI*)((IloAnySetI*)_impl)->getPrevious(s.getImpl(),n));
	}
	using IloSymbolSetBase::getNextC;

	IloSymbol getNextC(const char* value, IloInt n=1) const{
		IloAssert(getImpl() != 0, "IloSet: Using empty handle");
		IloSymbol s = getImpl()->getEnv()->makeSymbol(value);
		return IloSymbol((IloSymbolI*)((IloAnySetI*)_impl)->getNextC(s.getImpl(),n));
	}
	using IloSymbolSetBase::getPreviousC;

	IloSymbol getPreviousC(const char* value, IloInt n=1) const{
		IloAssert(getImpl() != 0, "IloSet: Using empty handle");
		IloSymbol s = getImpl()->getEnv()->makeSymbol(value);
		return IloSymbol((IloSymbolI*)((IloAnySetI*)_impl)->getPreviousC(s.getImpl(),n));
	}
};


typedef IloSymbolSet::Iterator IloSymbolSetIterator;

ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, const IloSymbolSet& set);


#endif
