// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __XML_ilowriterH
#define __XML_ilowriterH

#if !defined(__CONCERT_iloenvH)
#include <ilconcert/iloenv.h>
#endif

#if !defined(__CONCERT_ilosolutionH)
#include <ilconcert/ilosolution.h>
#endif

#if !defined(__CONCERT_ilosetH)
#include <ilconcert/iloset.h>
#endif

#if !defined(__CONCERT_iloanyH)
#include <ilconcert/iloany.h>
#endif
#if !defined(__CONCERT_ilomapH)
#include <ilconcert/ilomap.h>
#endif

#if !defined(__XML_iloxmlexceptionH)
#include <ilconcert/ilxml/iloxmlexception.h>
#endif

#if !defined(__XML_iloxmlhashH)
#include <ilconcert/ilxml/iloxmlhash.h>
#endif

#ifdef ILO_RS6000
void* operator new (size_t sz, const IloEnv& env);
void* operator new[] (size_t s, const IloEnv& env);
#endif

#if !defined(__XML_xmlH)
#include <ilconcert/ilxml/iloxml.h>
#endif
#if !defined(__XML_listH)
#include <ilconcert/ilxml/ilolist.h>
#endif
#if !defined(__XML_memoryH)
#include <ilconcert/ilxml/ilomemory.h>
#endif
#if !defined(__XML_poolH)
#include <ilconcert/ilxml/ilopool.h>
#endif

#if defined(IL_STD)
IL_STDUSE
#endif /* !IL_STD */

const IloInt IloNoRoundingMode = -1;

class IloXmlWriterI;
class IloXmlContextI;

class IloMapIntIndexI;
class IloMapIntExprIndexI;
class IloMapNumIndexI;
class IloMapNumExprIndexI;


class ILO_EXPORTED IloXmlWriter {
private:
  IloXmlWriterI* _impl;
public:

  void end();

  IloXmlIdManager& getIdManager();

  const IloXmlWriterI* getImpl();

  IloXmlWriter(IloEnv env, const char* rootTag, const char* fileName=0);

  IloXmlWriter(IloXmlWriterI* impl): _impl(impl) {};

  IloBool isSerialized(IloInt id);

  IloIntArray* getSerialized();

  IloIntArray* getSolutionSerialized();

  IloBool isSolutionSerialized(IloInt id);


  void deleteAllocatedMemory(char* pointer);

  void setNumPrecision(const IloInt num);
  IloInt getNumPrecision();

  IloInt string2Int(const char* str);

  void deleteAllocatedMemory(const char* pointer);

  void setfileName(const char* fileName);

  const char* getfileName();

  IloXmlElement* getRoot();

  IloEnv getEnv();

  IloEnvI* getEnvImpl();

  const char* Int2String(const IloInt number);

  const char* Num2String(const IloNum number);

  const char* IntArray2String(const IloIntArray intArray);

  const char* NumArray2String(const IloNumArray numArray);


 const char* IntSet2String(const IloIntSet intSet);

  const char* NumSet2String(const IloNumSet numSet);

  void addElement(IloXmlElement* element);

  void addSubElement(IloXmlElement* element, IloXmlElement* subElement);

  void addAttribute(IloXmlElement* element, const char* attribute, const char* value);

  IloXmlElement* createElement(const char* element);

  void addComment(IloXmlElement* element,const char* comment);

  void addText(IloXmlElement* element, const char* text);

  void addCData(IloXmlElement* element, const char* CData);

  IloBool writeDocument();
};


class ILO_EXPORTED IloWriterError : public IloException {
public:
  IloWriterError(const char* message);
  virtual ~IloWriterError();
};


class IloXmlWriterI : public IloEnvObjectI  {
private:
  int _numPrecision;
  char* _fileName;
  IloXmlDocument* _document;
  IloXmlElement* _root;
  IloXmlIdManager _idManager;

private:
  char* precision(double r);
  IloBool checkIfSolutionObject(IloInt id);

public:
  ~IloXmlWriterI();

  IloXmlIdManager& getIdManager() {
        return _idManager;
  }

  IloXmlWriterI(IloEnv env, const char* rootTag, const char* fileName=0);

  void deleteAllocatedMemory(char* pointer);

  void setNumPrecision(const IloInt num);
  IloInt getNumPrecision();


  IloIntArray* getSerialized();
  IloIntArray* getSolutionSerialized();

  IloBool isSolutionSerialized(IloInt id);
  IloBool isSerialized(IloInt id);

  IloInt string2Int(const char* string);

  void deleteAllocatedMemory(const char* pointer);

  void setfileName(const char* fileName);

  const char* getfileName();

  IloXmlElement* getRoot();

  const char* Int2String(const IloInt number);
  const char* Num2String(const IloNum number);
  const char* IntArray2String(const IloIntArray intArray);
  const char* NumArray2String(const IloNumArray numArray);

  const char* IntSet2String(const IloIntSet intSet);
  const char* NumSet2String(const IloNumSet numSet);

  void addElement(IloXmlElement* element);

  void addSubElement(IloXmlElement* element, IloXmlElement* subElement);

  void addAttribute(IloXmlElement* element, const char* attribute, const char* value);

  IloXmlElement* createElement(const char* element);

  void addComment(IloXmlElement* element,const char* comment);

  void addText(IloXmlElement* element, const char* text);

  void addCData(IloXmlElement* element, const char* CData);

  IloBool writeDocument();
  IloBool writeMap(IloXmlContextI* context, IloXmlElement* element, IloIntMap map);
  IloBool writeMap(IloXmlContextI* context, IloXmlElement* element, IloIntExprMap map);
  IloBool writeMap(IloXmlContextI* context, IloXmlElement* element, IloNumMap map);
  IloBool writeMap(IloXmlContextI* context, IloXmlElement* element, IloNumExprMap map);
};
#endif
