// --------------------------------------------------------------------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilodatacolumnH
#define __CONCERT_ilodatacolumnH

#if !defined(__CONCERT_ilocollectionH)
#include <ilconcert/ilocollection.h>
#endif
class IloIntDataColumnI;
class IloNumDataColumnI;
class IloAnyDataColumnI;

/**
* @group optim.concert.tuple
*
* <p>An instance of this class offers a convenient way to represent a set of integer
* values as a constrained variable in Concert Technology.</p>
*
*/
class IloIntDataColumn : public IloIntCollection {
	IloIntDataColumnI* _impl;
public:
	/**
	* <p>This constructor creates a handle to a set of numeric values from its
	* implementation object.</p>
	*/
	IloIntDataColumn(IloIntDataColumnI* impl=0);

	/**
	* <p>This member function returns a pointer to the implementation object of the
	* invoking set.</p>
	*/
	IloIntDataColumnI* getImpl() const;

	/**
	* <p>This member function returns an integer indicating the size of the invoking
	* set (that is, how many elements it contains).</p>
	*/
	IloInt getSize() const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>elt</code> is an element of the invoking set. The value one
	* indicates that the invoking set contains <code>elt</code>; the value zero
	* indicates that the invoking set does not contain <code>elt</code>.</p>
	*/
	IloBool contains(IloInt elt) const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>set</code> contains the invoking set. The value one indicates
	* that the invoking set contains all the elements of set, and that the
	* intersection of the invoking set with <code>set</code> is precisely
	* <code>set</code>. The value zero indicates that the intersection of the
	* invoking set and <code>set</code> is not precisely <code>set</code>.</p>
	*/
	IloBool contains(IloIntDataColumn set) const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>set</code> intersects the invoking set. The value one
	* indicates that the intersection of <code>set</code> and the invoking set is
	* not empty (at least one element in common); the value zero indicates that
	* the intersection of <code>set</code> and the invoking set is empty (no
	* elements in common).</p>
	*/
	IloBool intersects(IloIntDataColumn set) const;

	/**
	* @group optim.concert.tuple
	*
	* <p>An instance of the nested class <code>IloIntDataColumn::Iterator</code> is an
	* iterator that traverses the elements of a finite set of numeric values (an
	* instance of <code><cpp-link href='IloIntDataColumn'/></code>).</p>
	*
	* @see IloIntDataColumn
	*/
	class Iterator : public IloIntDefaultDataIterator {
	public:
		/** <p> . </p> */
		Iterator(const IloIntDataColumnI* coll);
		/** <p> . </p> */
		Iterator(const IloIntDataColumn coll);
		/** <p> . </p> */
		Iterator(IloGenAlloc* heap, const IloIntDataColumnI* coll);
		/** <p> . </p> */
		virtual ~Iterator();
	};
};

/**
* @group optim.concert.tuple
* <p> Shortcut for the iterator </p> **/
typedef IloIntDataColumn::Iterator IloIntDataColumnIterator;


/**
* @group optim.concert.tuple
*
* <p>An instance of this class offers a convenient way to represent a set of integer
* values as a constrained variable in Concert Technology.</p>
*
*/
class IloNumDataColumn : public IloNumCollection {
	IloNumDataColumnI* _impl;
public:
	/**
	* <p>This constructor creates a handle to a set of numeric values from its
	* implementation object.</p>
	*/
	IloNumDataColumn(IloNumDataColumnI* impl=0);

	/**
	* <p>This member function returns a pointer to the implementation object of the
	* invoking set.</p>
	*/
	IloNumDataColumnI* getImpl() const;

	/**
	* <p>This member function returns an integer indicating the size of the invoking
	* set (that is, how many elements it contains).</p>
	*/
	IloInt getSize() const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>elt</code> is an element of the invoking set. The value one
	* indicates that the invoking set contains <code>elt</code>; the value zero
	* indicates that the invoking set does not contain <code>elt</code>.</p>
	*/
	IloBool contains(IloNum elt) const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>set</code> contains the invoking set. The value one indicates
	* that the invoking set contains all the elements of set, and that the
	* intersection of the invoking set with <code>set</code> is precisely
	* <code>set</code>. The value zero indicates that the intersection of the
	* invoking set and <code>set</code> is not precisely <code>set</code>.</p>
	*/
	IloBool contains(IloNumDataColumn set) const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>set</code> intersects the invoking set. The value one
	* indicates that the intersection of <code>set</code> and the invoking set is
	* not empty (at least one element in common); the value zero indicates that
	* the intersection of <code>set</code> and the invoking set is empty (no
	* elements in common).</p>
	*/
	IloBool intersects(IloNumDataColumn set) const;

	/**
	* @group optim.concert.tuple
	*
	* <p>An instance of the nested class <code>IloNumDataColumn::Iterator</code> is an
	* iterator that traverses the elements of a finite set of numeric values (an
	* instance of <code><cpp-link href='IloNumDataColumn'/></code>).</p>
	*
	* @see IloNumDataColumn
	*/
	class Iterator : public IloNumDefaultDataIterator {
	public:
		/** <p> . </p> */
		Iterator(const IloNumDataColumnI* coll);
		/** <p> . </p> */
		Iterator(const IloNumDataColumn coll);

		/** <p> . </p> */
		Iterator(IloGenAlloc* heap, const IloNumDataColumnI* coll);
		/** <p> . </p> */
		virtual ~Iterator();
	};
};

/**
* @group optim.concert.tuple
* <p> Shortcut for the iterator </p> **/
typedef IloNumDataColumn::Iterator IloNumDataColumnIterator;



/**
* @group optim.concert.tuple
*
* <p>An instance of this class offers a convenient way to represent a set of integer
* values as a constrained variable in Concert Technology.</p>
*
*/
class IloAnyDataColumn : public IloAnyCollection {
	IloAnyDataColumnI* _impl;
public:
	/**
	* <p>This constructor creates a handle to a set of numeric values from its
	* implementation object.</p>
	*/
	IloAnyDataColumn(IloAnyDataColumnI* impl=0);

	/**
	* <p>This member function returns a pointer to the implementation object of the
	* invoking set.</p>
	*/
	IloAnyDataColumnI* getImpl() const;

	/**
	* <p>This member function returns an integer indicating the size of the invoking
	* set (that is, how many elements it contains).</p>
	*/
	IloInt getSize() const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>elt</code> is an element of the invoking set. The value one
	* indicates that the invoking set contains <code>elt</code>; the value zero
	* indicates that the invoking set does not contain <code>elt</code>.</p>
	*/
	IloBool contains(IloAny elt) const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>set</code> contains the invoking set. The value one indicates
	* that the invoking set contains all the elements of set, and that the
	* intersection of the invoking set with <code>set</code> is precisely
	* <code>set</code>. The value zero indicates that the intersection of the
	* invoking set and <code>set</code> is not precisely <code>set</code>.</p>
	*/
	IloBool contains(IloAnyDataColumn set) const;

	/**
	* <p>This member function returns a Boolean value (zero or one) that indicates
	* whether <code>set</code> intersects the invoking set. The value one
	* indicates that the intersection of <code>set</code> and the invoking set is
	* not empty (at least one element in common); the value zero indicates that
	* the intersection of <code>set</code> and the invoking set is empty (no
	* elements in common).</p>
	*/
	IloBool intersects(IloAnyDataColumn set) const;

	/**
	* @group optim.concert.tuple
	*
	* <p>An instance of the nested class <code>IloAnyDataColumn::Iterator</code> is an
	* iterator that traverses the elements of a finite set of numeric values (an
	* instance of <code><cpp-link href='IloAnyDataColumn'/></code>).</p>
	*
	* @see IloAnyDataColumn
	*/
	class Iterator : public IloAnyDefaultDataIterator {
	public:
		/** <p> . </p> */
		Iterator(const IloAnyDataColumnI* coll);
		/** <p> . </p> */
		Iterator(const IloAnyDataColumn coll);

		/** <p> . </p> */
		Iterator(IloGenAlloc* heap, const IloAnyDataColumnI* coll);
		/** <p> . </p> */
		virtual ~Iterator();
	};
};

/**
* @group optim.concert.tuple
* <p> Shortcut for the iterator </p> **/
typedef IloAnyDataColumn::Iterator IloAnyDataColumnIterator;


#endif
