// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplebufferH
#define __CONCERT_ilotuplebufferH

#if !defined(__CONCERT_ilocollectionH)
# include <ilconcert/ilocollection.h>
#endif

#if !defined(__CONCERT_ilolinkedlistH)
# include <ilconcert/ilolinkedlist.h>
#endif

class IloTupleI;
class IloTupleCollectionI;
class IloTupleSchemaI;

/**
* @group optim.concert.tuple
* @show internal
* <p> </p>
**/
class IloTuplePath : public IloEnvObjectI {
public:
	/**
	* @group optim.concert.tuple
	* @show internal
	* <p> TO BE DOCUMENTED?? </p>
	**/
	class IloTupleCell {
	public:
		union Value{
			IloInt _int;
			IloNum _num;
			IloAny _any;
		};
	private:
		IloDataType _type;
		Value _value;		
	public:
		IloDataType getType() const { return _type; }
		Value getValue() const { return _value; }
		/** <p> . </p> */
		IloInt* getIntAddress() { return &(_value._int); }
		/** <p> . </p> */
		IloNum* getNumAddress() { return &(_value._num); }
		/** <p> . </p> */
		IloAny* getAnyAddress() { return &(_value._any); }

		/** <p> . </p> */
		IloBool isInt() const{ return (_type == IntDataColumn);	}
		/** <p> . </p> */
		IloBool isNum() const{ return (_type == NumDataColumn);	}
        /** <p> . </p> */
		IloBool isAny() const{ return (_type == AnyDataColumn); }

		/** <p> . </p> */
		IloInt getIntValue() const{
			if (this->isInt())
				return _value._int; 
			else if (this->isNum() && IloNumIsInteger(_value._num))
				return IloNumToInt(_value._num);
			throw IloWrongUsage("IloTuplePath::IloTupleCell::getIntValue() type not int nor num");
			ILOUNREACHABLE(return 0;)
		}
		/** <p> . </p> */
		IloNum getNumValue() const{
			if (this->isNum())
				return _value._num;
			else if (this->isInt())
				return _value._int; 
			throw IloWrongUsage("IloTuplePath::IloTupleCell::getNumValue() type not num nor int");
			ILOUNREACHABLE(return 0;)
		}
		/** <p> . </p> */
		IloAny getAnyValue() const{
			return _value._any;
		}

		/** <p> . </p> */
		IloTupleCell(IloInt val) {
			_type = IntDataColumn;
			_value._int = val;
		}
		/** <p> . </p> */
		IloTupleCell(IloNum val){
			_type = NumDataColumn; 
			_value._num = val;
		}
		/** <p> . </p> */
		IloTupleCell(IloAny val){
			_type = AnyDataColumn;
			_value._any = val;
		}
		/** <p> . </p> */
		void setValue(IloInt val){
			_type = IntDataColumn;
			_value._int = val;
		}
		/** <p> . </p> */
		void setValue(IloNum val){
			_type = NumDataColumn;
			_value._num = val;
		}
		/** <p> . </p> */
		void setValue(IloAny val){
			_type = AnyDataColumn;
			_value._any = val;
		}
		~IloTupleCell(){}

		IloBool operator!=(const IloTupleCell& other) {
			if (_type != other.getType()) return IloTrue;
			switch(_type){
			case IntDataColumn:
				return _value._int != other._value._int;
			break;
			case NumDataColumn:
				return _value._num != other._value._num;
			break;
			default:
				return _value._any != other._value._any;
			break;
			}
		}
	};
public:
  enum IloTupleRequestType {
    Equality = 0,
    LowerBound = 32,
    UpperBound = 64
  };

private:
	IloTupleCell _cell;
	IloIntArray _path;
	IloTupleRequestType _reqType;
public:
	/** <p> . </p> */
	IloTupleCell& getCell() { return _cell; }
	/** <p> . </p> */
	IloIntArray getPath() const { return _path; }
public:
	~IloTuplePath(){ _path.end(); }
	void setValue(IloTupleCell cell){ _cell = cell;}
	/** <p> . </p> */
	void setValue(IloInt value){ getCell().setValue(value);	}
	/** <p> . </p> */
	void setValue(IloNum value){ getCell().setValue(value);	}
	/** <p> . </p> */
	void setValue(IloAny value){ getCell().setValue(value);	}
	/** <p> . </p> */
	void setValue(IloDiscreteDataCollection value){ getCell().setValue((IloAny)value.getImpl()); }
	/** <p> . </p> */
	IloTupleRequestType getRequestType() const {
		return _reqType;
	}
	/** <p> . </p> */
	IloBool isEquality() const {
		return _reqType==Equality;
	}
	/** <p> . </p> */
	IloBool isLowerBound() const {
		return _reqType==LowerBound;
	}
	/** <p> . </p> */
	IloBool isUpperBound() const {
		return _reqType==UpperBound;
	}

	IloTuplePath(IloIntArray path, IloTupleCell cell, IloTupleRequestType reqType = Equality): 
	IloEnvObjectI(path.getEnv().getImpl()), _cell(cell), _path(path), _reqType(reqType) {
	}
	/** <p> . </p> */
	IloTuplePath(IloEnv env, IloIntArray path, IloAny value, IloTupleRequestType reqType = Equality): 
	IloEnvObjectI(env.getImpl()), _cell(value), _path(path), _reqType(reqType) {
		IloTestAndRaise(path.getImpl(), "IloTuplePath: path is empty");
	}
	/** <p> . </p> */
	IloTuplePath(IloEnv env, IloIntArray path, IloNum value, IloTupleRequestType reqType = Equality): 
	IloEnvObjectI(env.getImpl()), _cell(value), _path(path), _reqType(reqType) {
		IloTestAndRaise(path.getImpl(), "IloTuplePath: path is empty");
	}
	/** <p> . </p> */
	IloTuplePath(IloEnv env, IloIntArray path, IloInt value, IloTupleRequestType reqType = Equality): 
	IloEnvObjectI(env.getImpl()), _cell(value), _path(path), _reqType(reqType) {
		IloTestAndRaise(path.getImpl(), "IloTuplePath: path is empty");
	}
};

typedef IloArray<IloTuplePath*> IloTuplePathArray;

class IloTupleCellArrayI : public IloEnvObjectI {
	IloInt _size;
	IloTuplePath::IloTupleCell* _values;
public:
	IloTupleCellArrayI(IloTupleCellArrayI* toCopy) : IloEnvObjectI(toCopy->getEnv()), _size(toCopy->getSize()), _values((IloTuplePath::IloTupleCell*)getEnv()->alloc( sizeof(IloTuplePath::IloTupleCell)*toCopy->getSize())){
		memcpy((void*)_values, (const void*)toCopy->getValues(), _size*sizeof(IloTuplePath::IloTupleCell));
	}
	IloTupleCellArrayI(IloEnv env, IloInt size) : IloEnvObjectI(env.getImpl()), _size(size), _values(0){
		if (_size) _values = (IloTuplePath::IloTupleCell*)getEnv()->alloc( sizeof(IloTuplePath::IloTupleCell)*size );
	}
	~IloTupleCellArrayI(){
		getEnv()->free(_values, sizeof(IloTuplePath::IloTupleCell)*_size);
		_values = 0;
		_size = 0;
	}
	const IloTuplePath::IloTupleCell& operator[] (IloInt i) const{
		IloAssert ( i>=0, "Index out of bounds operation: negative index");
		IloAssert ( i < _size, "X& IloArray::operator[] (IloInt i) : Out of bounds operation: index superior to size of array");
		return _values[i];
	}
	IloTuplePath::IloTupleCell& operator[] (IloInt i) {
		IloAssert ( i>=0, "Index out of bounds operation: negative index");
		IloAssert ( i < _size, "X& IloArray::operator[] (IloInt i) : Out of bounds operation: index superior to size of array");
		return _values[i];
	}
	void add(IloTuplePath::IloTupleCell i){
		IloTuplePath::IloTupleCell* temp = (IloTuplePath::IloTupleCell*)getEnv()->alloc( sizeof(IloTuplePath::IloTupleCell)*(_size+1) );
		for (IloInt j=0; j< _size; j++){
			IloTuplePath::IloTupleCell cell = _values[j];
			temp[j] = cell;
		}
		temp[_size] = i;
		getEnv()->free(_values, sizeof(IloTuplePath::IloTupleCell)*_size);
		_values = temp;
		_size++;
	}
	void add(IloTupleCellArrayI* array){
		IloTuplePath::IloTupleCell* temp = (IloTuplePath::IloTupleCell*)getEnv()->alloc( sizeof(IloTuplePath::IloTupleCell)*(_size+ array->getSize()) );
		for (IloInt j=0; j< _size; j++){
			IloTuplePath::IloTupleCell cell = _values[j];
			temp[j] = cell;
		}
		for (IloInt i=0; i< array->getSize(); i++){
			IloTuplePath::IloTupleCell cell = array->getValues()[i];
			temp[_size+i] = cell;
		}
		getEnv()->free(_values, sizeof(IloTuplePath::IloTupleCell)*_size);
		_values = temp;
		_size+=array->getSize();
	}
	IloInt getSize() const { return _size;} 
	IloTupleCellArrayI* copy(){
		return new (getEnv()) IloTupleCellArrayI(this);
	}

    void setSize(IloInt size) {
		IloTuplePath::IloTupleCell* temp = (IloTuplePath::IloTupleCell*)getEnv()->alloc( sizeof(IloTuplePath::IloTupleCell)*size );
		for (IloInt j=0; j< _size && j < size; j++){
			IloTuplePath::IloTupleCell cell =_values[j];
			temp[j] = cell;
		}
		getEnv()->free(_values, sizeof(IloTuplePath::IloTupleCell)*_size);
		_values = temp;
		_size = size;
    }
    void clear() {
		getEnv()->free(_values, sizeof(IloTuplePath::IloTupleCell)*_size);
		_values = 0;
		_size = 0;
    }
	IloTuplePath::IloTupleCell* getValues(){ return _values; }
};

class IloTupleCellArray {
    IloTupleCellArrayI* _impl;
public:
	IloTupleCellArray(IloEnv env, IloInt size = 0) : _impl(new (env) IloTupleCellArrayI(env, size)){};
	IloTupleCellArray() : _impl(0){};
	IloTupleCellArray(IloTupleCellArrayI* impl) : _impl(impl){};
	IloTupleCellArrayI* getImpl() const{
		return _impl;
	}
	const IloTuplePath::IloTupleCell& operator[] (IloInt i) const {
		return _impl->operator[](i);
	}
	IloTuplePath::IloTupleCell& operator[] (IloInt i) {
		return _impl->operator[](i);
	}

	void add(IloTuplePath::IloTupleCell i){
		_impl->add(i);
	}
	void add(IloTupleCellArray array){
		_impl->add(array.getImpl());
	}
	void end(){
		if (_impl) delete _impl;
		_impl = 0;
	}
	IloInt getSize() const { return _impl->getSize(); } 

	IloBool isHomogeneous() const;
	void convertTo(const IloTupleCollectionI* set, const IloTupleSchemaI* schema =0);
	void display(ILOSTD(ostream)& os) const;
	IloTupleCellArray makeSlice(IloInt index, IloInt size);
	IloTupleCellArray fillSlice(IloTupleCellArray array, IloInt index, IloInt size);
	void endAll();
	IloTupleCellArrayI* copy() const{
		return _impl->copy();
	}
	IloEnv getEnv() const{ return _impl->getEnv(); }
    void setSize(IloInt size) {
		_impl->setSize(size);
    }
    void clear() {
		_impl->clear();
	}
};

IloBool operator==(const IloTupleCellArray cells1, const IloTupleCellArray cells2);


typedef IloSimpleLinkedList<IloTuplePath*> IloTuplePathListBase;

/**
* @group optim.concert.tuple
* @show internal
* <p> </p>
**/
class IloTuplePathList : public IloSimpleLinkedList<IloTuplePath*> {
public:
	/** <p> . </p> */
	using IloSimpleLinkedList<IloTuplePath*>::add;
	/** <p> . </p> */
	IloTuplePathList(IloEnv env) : IloTuplePathListBase(env)  {}
	~IloTuplePathList();
};


//-------------------------------------

/**
* @group optim.concert.tuple
* @show internal
* <p> TO BE DOCUMENTED?? </p>
**/
class IloTuplePathBufferI : public IloRttiEnvObjectI{
	ILORTTIDECL
	friend class IloTupleBufferI;
public:
	IloTuplePath* add(IloIntArray path, IloInt value);
	IloTuplePath* add(IloIntArray path, IloNum value);
	IloTuplePath* add(IloIntArray path, IloAny value);

	IloTuplePath* modify(IloTuplePath* tuple, IloInt value);
	IloTuplePath* modify(IloTuplePath* tuple, IloNum value);
	IloTuplePath* modify(IloTuplePath* tuple, IloAny value);

protected:
	IloBool _isOrd;
	IloTuplePathList _list;
	IloTuplePath* contains(IloIntArray array);
	IloTuplePath* contains(IloInt size, IloInt* path);
public:
	IloBool isOrd() const{ return _isOrd; }
	void setIsOrd(IloBool flag) { _isOrd = flag; }
	/** <p> . </p> */
	IloTuplePathBufferI(IloEnv env);
	virtual ~IloTuplePathBufferI();

	/** <p> . </p> */
	IloBool isEmpty() const { return _list.getSize() == 0; }
	/** <p> . </p> */
	IloBool isSingleton() const { return _list.getSize() == 1; }

	/** <p> . </p> */
	IloInt getSize() const { return _list.getSize(); }
	/** <p> . </p> */
	IloTuplePathList& getList() { return _list; }

	/** <p> . </p> */
	void clear() { 
		for (IloInt i=0; i< _list.getSize(); i++){
			IloTuplePath* path = _list.getObject(i);
			delete path; path=0;
		}
		_list.clear(); 
	}
	/** <p> . </p> */
	void import(IloTuplePathBufferI* select, IloInt prefix =-1);

	///** <p> . </p> */
	void import(IloInt idx1, IloTupleI* value);
	/** <p> . </p> */
	void import(IloIntArray path, IloTupleI* value);
	/** <p> . </p> */
	void import(IloInt pathSize, IloInt* path, IloTupleI* value);

	/** <p> . </p> */
	IloTuplePath* addOnce(IloInt columnIndex, IloInt value);
	/** <p> . </p> */
	IloTuplePath* addOnce(IloInt columnIndex, IloNum value);
	/** <p> . </p> */
	IloTuplePath* addOnce(IloInt columnIndex, IloAny value);

	/** <p> . </p> */
	IloTuplePath* addOnce(IloIntArray path, IloBool isArrayInternal, IloInt value);
	/** <p> . </p> */
	IloTuplePath* addOnce(IloIntArray path, IloBool isArrayInternal, IloNum value);
	/** <p> . </p> */
	IloTuplePath* addOnce(IloIntArray path, IloBool isArrayInternal, IloAny value);

	/** <p> . </p> */
	IloTuplePath* addOnce(IloInt pathSize, IloInt* path, IloInt value);
	/** <p> . </p> */
	IloTuplePath* addOnce(IloInt pathSize, IloInt* path, IloNum value);
	/** <p> . </p> */
	IloTuplePath* addOnce(IloInt pathSize, IloInt* path, IloAny value);

	/** <p> . </p> */
	virtual void display(ILOSTD(ostream)& os) const;
};


/**
* @group optim.concert.tuple
* @show internal
* <p>An instance of this class offers a convenient way to represent a buffer for a IloTuple.</p>
*/
class IloTuplePathBuffer {
protected:
	IloTuplePathBufferI* _impl;
public:
	/** <p> Returns the env </p> **/
	IloEnv getEnv(){
		return _impl->getEnv();
	}

	/** <p> . </p> */
	IloTuplePathList& getList(){
		return _impl->getList();
	}

	/** <p> Constructor from an implementation. </p> **/
	IloTuplePathBuffer(IloTuplePathBufferI* impl) : _impl(impl){ }

	/** <p> This member function returns a pointer to the implementation
	* object of the invoking extractable object.
	* This member function is useful when you need to be sure that
	* you are using the same copy of the invoking extractable object
	* in more than one situation. </p>
	**/
	IloTuplePathBufferI* getImpl() const {
		return _impl;
	}

	/** <p> This member function deletes the invoking data domain.
	* That is, it frees all the resources used by the invoking object.
	* After a call to this member function, you can not use the invoking
	*  extractable object again.</p>
	**/
	void end(){
		if (_impl) {
			delete _impl; _impl = 0;
		}
	}
	/** <p> Select <i>value</i> from <i>columnName</i> </p> **/
	IloTuplePathBuffer(IloEnv env);

	/** <p> Displays the query </p> **/
	void display(ILOSTD(ostream)& os) const{
		IloAssert(getImpl() != 0, "IloTuplePathBuffer: Using empty handle");
		_impl->display(os);
	}
	/** <p> Returns the size. </p> **/
	IloInt getSize() const {
		IloAssert(getImpl() != 0, "IloTuplePathBuffer: Using empty handle");
		return _impl->getSize();
	}
};

#endif
