// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplerequestH
#define __CONCERT_ilotuplerequestH

#if !defined(__CONCERT_ilotuplerequestiH)
# include <ilconcert/iltuple/ilotuplerequesti.h>
#endif

/**
* @group optim.concert.tuple
* @show internal
* <p>An instance of this class offers a convenient way to represent a selection in the IloTupleSet.</p>
*/
class IloTupleRequest {
protected:
	IloTupleRequestI* _impl;
public:
	/** <p> This member function deletes the invoking data domain.
	* That is, it frees all the resources used by the invoking object.
	* After a call to this member function, you can not use the invoking
	*  extractable object again.</p>
	**/
	void end(){
		if (_impl) {
			delete _impl; _impl = 0;
		}
	}

	/** <p> Displays the query </p> **/
	void display(ILOSTD(ostream)& os) const{
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		_impl->display(os);
	}

	/** <p> Makes a And jointure </p> **/
	IloTuplePath* add(IloIntArray path, IloInt value){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		return _impl->add(path, value);
	}
	/** <p> Makes a And jointure </p> **/
	IloTuplePath* add(IloIntArray path, IloNum value){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		return _impl->add(path, value);
	}
	/** <p> Makes a And jointure </p> **/
	IloTuplePath* add(IloIntArray path, IloAny value){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		return _impl->add(path, value);
	}
	/** <p> Makes a And jointure </p> **/
	IloTuplePath* add(IloIntArray path, IloSymbol value){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		return _impl->add(path, value);
	}

	/** <p> Makes a And jointure </p> **/
	IloTuplePath* add(IloIntArray path, IloIntCollection value){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		return _impl->add(path, value.getImpl());
	}
	/** <p> Makes a And jointure </p> **/
	IloTuplePath* add(IloIntArray path, IloNumCollection value){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		return _impl->add(path, value.getImpl());
	}
	/** <p> Makes a And jointure </p> **/
	IloTuplePath* add(IloIntArray path, IloAnyCollection value){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		return _impl->add(path, value.getImpl());
	}


	/** <p> . </p> */
	IloInt getSize() const {
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		return _impl->getSize();
	}

	/** <p> Returns the env </p> **/
	IloEnv getEnv(){
		return _impl->getEnv();
	}

	/** <p> TO BE DOCUMENTED ?? </p> **/
	IloTupleRequest copy() const;

	/** <p> Constructor from an implementation. </p> **/
	IloTupleRequest(IloTupleRequestI* impl) : _impl(impl){ }

	/** <p> This member function returns a pointer to the implementation
	* object of the invoking extractable object.
	* This member function is useful when you need to be sure that
	* you are using the same copy of the invoking extractable object
	* in more than one situation. </p>
	**/
	IloTupleRequestI* getImpl() const {
		return (IloTupleRequestI*)_impl;
	}

	/** <p> Constructor </p> **/
	IloTupleRequest(IloEnv env, IloTupleSet coll);


	/** <p> Creates the indexes </p> **/
	void createSelectIndexes(){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		((IloTupleRequestI*)_impl)->createSelectIndexes();
	}
	/** <p> Clears the indexes </p> **/
	void clearSelectIndexes(){
		IloAssert(getImpl() != 0, "IloTupleRequest: Using empty handle");
		((IloTupleRequestI*)_impl)->clearSelectIndexes();
	}
};

#endif

