// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplerequestiH
#define __CONCERT_ilotuplerequestiH

#if !defined(__CONCERT_ilostringH)
# include <ilconcert/ilostring.h>
#endif
#if !defined(__CONCERT_ilosymbolH)
# include <ilconcert/ilosymbol.h>
#endif
#if !defined(__CONCERT_ilotuplebufferH)
# include <ilconcert/iltuple/ilotuplebuffer.h>
#endif
#if !defined(__CONCERT_ilorequestcacheH)
# include <ilconcert/iltuple/ilorequestcache.h>
#endif


class IloTupleRequest;
class IloTupleRequestI;
class IloTupleSetI;
class IloTupleSet;
class IloTupleSchemaI;
class IloTupleSchema;
class IloTupleBuffer;
class IloTupleRequestChangeListenerI;


class IloIntFixedArrayI : public IloEnvObjectI {
	IloInt _size;
	IloInt* _values;
public:
	IloIntFixedArrayI(IloIntFixedArrayI* toCopy) : IloEnvObjectI(toCopy->getEnv()), _size(toCopy->getSize()), _values((IloInt*)getEnv()->alloc( sizeof(IloInt)*toCopy->getSize() )){
		memcpy((void*)_values, (const void*)toCopy->getValues(), _size*sizeof(IloInt));
	}

	IloIntFixedArrayI(IloEnv env, IloInt size) : IloEnvObjectI(env.getImpl()), _size(size), _values(0){
		if (_size) _values = (IloInt*)getEnv()->alloc( sizeof(IloInt)*size );
	}
	~IloIntFixedArrayI(){
		getEnv()->free(_values, sizeof(IloInt)*_size);
		_values = 0;
		_size = 0;
	}
	IloInt operator[] (IloInt i) const{
		IloAssert ( i>=0, "Index out of bounds operation: negative index");
		IloAssert ( i < _size, "X& IloArray::operator[] (IloInt i) : Out of bounds operation: index superior to size of array");
		return _values[i];
	}
	void setValue(IloInt index, IloInt value){
		IloAssert ( index>=0, "Index out of bounds operation: negative index");
		IloAssert ( index < _size, "X& IloArray::operator[] (IloInt i) : Out of bounds operation: index superior to size of array");
		_values[index] = value;
	}
	IloInt getSize() const { return _size;} 
	IloIntFixedArrayI* copy(){
		return new (getEnv()) IloIntFixedArrayI(this);
	}
	IloInt* getValues(){ return _values; }
};

class IloIntFixedArray {
    IloIntFixedArrayI* _impl;
public:
	IloIntFixedArray(IloEnv env, IloInt size = 0) : _impl(new (env) IloIntFixedArrayI(env, size)){};
	IloIntFixedArray() : _impl(0){};
	IloIntFixedArray(IloIntFixedArrayI* impl) : _impl(impl){};
	IloIntFixedArrayI* getImpl() const{
		return _impl;
	}
	IloInt operator[] (IloInt i) const {
		return _impl->operator[](i);
	}
	void setValue(IloInt index, IloInt value){
		_impl->setValue(index, value);
	}

	void end(){
		if (_impl) delete _impl;
		_impl = 0;
	}
	IloInt getSize() const { return _impl->getSize(); } 
	IloIntFixedArrayI* copy() const{
		return _impl->copy();
	}
	IloEnv getEnv() const{ return _impl->getEnv(); }
};
class IloFixedReqTypeArrayI : public IloEnvObjectI {
	IloInt _size;
	IloTuplePath::IloTupleRequestType* _values;
public:
	IloFixedReqTypeArrayI(IloEnv env, IloInt size) : IloEnvObjectI(env.getImpl()), _size(size), _values(0){
		if (_size) _values = (IloTuplePath::IloTupleRequestType*)getEnv()->alloc( sizeof(IloTuplePath::IloTupleRequestType)*size );
	}
	~IloFixedReqTypeArrayI(){
		getEnv()->free(_values, sizeof(IloTuplePath::IloTupleRequestType)*_size);
	}
	IloTuplePath::IloTupleRequestType operator[] (IloInt i) const {
		IloAssert ( i>=0, "Index out of bounds operation: negative index");
		IloAssert ( i < _size, "X& IloArray::operator[] (IloInt i) : Out of bounds operation: index superior to size of array");
		return _values[i];
	}
	void setValue(IloInt index, IloTuplePath::IloTupleRequestType value){
		IloAssert ( index>=0, "Index out of bounds operation: negative index");
		IloAssert ( index < _size, "X& IloArray::operator[] (IloInt i) : Out of bounds operation: index superior to size of array");
		_values[index] = value;
	}

	IloInt getSize() const { return _size;} 
	IloFixedReqTypeArrayI* copy();
	IloTuplePath::IloTupleRequestType* getValues(){ return _values; }
};

class IloFixedReqTypeArray {
    IloFixedReqTypeArrayI* _impl;
public:
	IloFixedReqTypeArray(IloEnv env, IloInt size = 0) : _impl(new (env) IloFixedReqTypeArrayI(env, size)){};
	IloFixedReqTypeArray() : _impl(0){};
	IloFixedReqTypeArray(IloFixedReqTypeArrayI* impl) : _impl(impl){};
	IloFixedReqTypeArrayI* getImpl() const{
		return _impl;
	}
	IloTuplePath::IloTupleRequestType operator[] (IloInt i) const {
		return _impl->operator[](i);
	}

	void end(){
		if (_impl) delete _impl;
		_impl = 0;
	}
	IloInt getSize() const { return _impl->getSize(); } 
	IloFixedReqTypeArrayI* copy() const{
		return _impl->copy();
	}
	IloEnv getEnv() const{ return _impl->getEnv(); }
	void setValue(IloInt index, IloTuplePath::IloTupleRequestType value){
		_impl->setValue(index, value);
	}
};

/**
* @group optim.concert.tuple
* @show internal
* <p> IloRequestSignatureI </p> **/
class IloRequestSignatureI : public IloEnvObjectI {
private:
	IloIntFixedArray _signature;
    IloFixedReqTypeArray _reqTypes;
public:
	/** <p> . </p> */
	IloInt getSize() const { return _signature.getSize(); }
	IloIntFixedArray getArray() const { return _signature; }
	/** <p> . </p> */
	IloFixedReqTypeArray getReqTypes() const { return _reqTypes; }
	/** <p> . </p> */
	IloRequestSignatureI(IloTupleRequestI* req);
	~IloRequestSignatureI();
	/** <p> . </p> */
	IloBool equal(IloRequestSignatureI* sign) const;
};


/**
* @group optim.concert.tuple
* @show internal
* <p>An instance of this class offers a convenient way to represent a selection in the IloTupleSet.</p>
*/
class IloTupleRequestI : public IloEnvObjectI {
  friend class IloTupleRequestChangeListenerI;
private:
	enum RequestType{
		IntRequest,
		NumRequest,
		AnyRequest,
		IntAnyRequest,
		MixedRequest,
		Undefined
	};
	enum Check{
		ToBeChecked = -1,
		CheckedTrue = 0,
		CheckedFalse = 1
	};

  RequestType _reqType;
  IloTuplePathList _list;
  IloTupleSchemaI* _schema;
  IloTupleSetI* _coll;
  IloTupleRequestCacheI* _cache;
  IloTupleRequestUnaryCacheI* _unaryCache;
  Check  _findRequest;
  Check  _subTupleRequest;
  Check _containsAllKeys;
  IloTupleRequestI* _subRequest;
  IloTupleRequestI* _keyRequest;
  IloTupleRequestI* _nonKeyRequest;
  Check _isOrd;
  IloBool _mustClear;

  IloTupleRequestChangeListenerI* _changeListener;
public:
  void  setMustClear(IloBool flag) { _mustClear = flag; }
  IloBool mustClear() const { return _mustClear; }
	void checkRequest(){
		this->checkIsFindRequest();
		this->checkIsSubTupleRequest();
		this->checkContainsAllKeys();
		this->checkIsOrd();
		if (isIntQuery()) { _reqType = IntRequest; }
		else if (isAnyQuery()) { _reqType = AnyRequest; }
		else if (isNumQuery()) { _reqType = NumRequest; }
		else if (isIntAnyQuery()) { _reqType = IntAnyRequest; }
		else _reqType = MixedRequest;
	}
	RequestType getRequestType() const { return _reqType; }
	void setRequestType(RequestType type){ _reqType = type; }
	void setRequestType(IloTupleRequestI* req){
		if (_reqType == Undefined || _reqType == MixedRequest) return;
		req->setRequestType(_reqType);
	}
	IloInt getColumnIndex(const char* name) const;
	IloIntArray makeColumnIndexArray(IloStringArray name) const;

	IloTuplePath* addWithoutCopy(IloIntArray path, IloInt value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	IloTuplePath* addWithoutCopy(IloIntArray path, IloNum value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	IloTuplePath* addWithoutCopy(IloIntArray path, IloAny value);

	void buildIntCaches();
	void buildNumCaches();
	void buildAnyCaches();
	void buildCommonCaches();

	void buildCache();
protected:

  friend class IloTupleIterator;
  void setCollection(IloTupleSetI* coll) { 
	  if (_coll && _coll!=coll){
		  this->deleteAllCaches();
	  }
	  if (_subRequest) _subRequest->setCollection(coll);
	  if (_keyRequest) _keyRequest->setCollection(coll);
	  if (_nonKeyRequest) _nonKeyRequest->setCollection(coll);
	  _coll = coll;
  }
  void deleteAllCaches();

public:
	IloTupleSchemaI* getSchema() const{
		return _schema;
	}
  IloInt getInternalId(IloIntArray path) const;
  void checkIsFindRequest();
  void checkIsOrd();
  IloBool isOrd() const { 
	  return (_isOrd == CheckedTrue) ? IloTrue : IloFalse;
  }
  void  setIsOrd(Check flag) { _isOrd = flag; }
  IloBool isFindRequest() const {
	  return (_findRequest == CheckedTrue) ? IloTrue : IloFalse;
  }
  void checkIsSubTupleRequest();
  IloBool isSubTupleRequest() const {
	  return (_subTupleRequest == CheckedTrue) ? IloTrue : IloFalse;
  }

  void checkContainsAllKeys();
  IloBool containsAllKeys() const {
	  return (_containsAllKeys == CheckedTrue) ? IloTrue : IloFalse;
  }
  void setContainsAllKeys(Check flag){ _containsAllKeys = flag;}

  IloTupleRequestI* getOrMakeKeyRequest();
  IloTupleRequestI* getOrMakeNonKeyRequest();

  IloTupleRequestI* getOrMakeSubRequest();
	/** <p> . </p> */
	void createSelectIndexes();
	/** <p> . </p> */
	void clearSelectIndexes();

	/** <p> . </p> */
	IloTuplePathList& getList();

	/** <p> . </p> */
	void clear();
	/** <p> . </p> */
	void import(IloTupleRequestI* select);


	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloInt value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloNum value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloSymbol value);

	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloIntCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloNumCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloAnyCollectionI* value);

	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloInt value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloNum value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloSymbol value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloIntCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloNumCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloAnyCollectionI* value);

	/** <p> . </p> */
	IloTuplePathArray addRecursive(IloTupleSchemaI* schema, IloIntArray path);
	IloTuplePathArray addRecursive(IloTupleSchemaI* schema, IloInt size, IloInt* path);

  /** <p> . </p> */
	IloInt getSize() const;

	/** <p> . </p> */
	IloTupleSetI* getCollection() const { return _coll; }
	virtual ~IloTupleRequestI();

	/** <p> . </p> */
	IloBool isEmpty() const { return _list.getSize() == 0; }
	/** <p> . </p> */
	IloBool isSingleColumn() const { return _list.getSize() == 1; }

	/** <p> . </p> */
	IloBool isIntQuery() const;
	/** <p> . </p> */
	IloBool isNumQuery() const;
	/** <p> . </p> */
	IloBool isAnyQuery() const;
	IloBool isIntAnyQuery() const;

	/** <p> . </p> */
	IloTuplePath* getFirst() const { return _list.firstObject(); }

	/** <p> . </p> */
	IloBool storeCache(IloIntArray slice);
	/** <p> . </p> */
	IloIntArrayI* getCache() {
		return ((_cache == 0) ? (IloIntArrayI*)0 : _cache->execute());
	}

	IloInt getUnaryCache() {
		return ((_unaryCache == 0) ? (IloInt)-1 : _unaryCache->execute());
	}
	/** <p> . </p> */
	IloTupleRequestI(IloEnv env, IloTupleSetI* coll);
	/** <p> . </p> */
	IloTupleRequestI(IloEnv env, IloTupleSchemaI* schema);

	/** <p> . </p> */
	virtual void display(ILOSTD(ostream)& os) const;
	/** <p> . </p> */
	IloBool contains(IloInt value);
	/** <p> . </p> */
	IloBool contains(IloNum value);
	/** <p> . </p> */
	IloBool contains(IloAny value);
};

#endif

