// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __XML_iloxmllintermH
#define __XML_iloxmllintermH



#if !defined(__XML_iloxmlabstractH)
#include <ilconcert/iloxmlabstract.h>
#endif

class IloIntLinTermI;
class IloNumLinTermI;

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_IntLinTerm</code> 
 * makes it possible to serialize an IloIntLinTermI in XML. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_IntLinTerm : public IloXmlInfo {
	
public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_IntLinTerm::getTagName(); }
  IloXmlInfo_IntLinTerm(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloIntLinTermI* intlintermI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloIntLinTermI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);

protected:
  IloIntExpr getIntExprNonLinear(IloXmlReader reader, IloXmlElement* element);
  void getExprConstant(IloXmlReader reader, IloXmlElement* element, IloInt& constant);
  void getExprLinear(IloXmlReader reader, IloXmlElement* element, IloNumArray&Coef, IloNumVarArray& varArray);

  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloRtti* exprI){
    return getTagElement(writer, (IloIntLinTermI*)exprI);
  }
  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloIntLinTermI* exprI);


};



/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_NumLinTerm</code> 
 * makes it possible to serialize an IloNumLinTermI in XML. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_NumLinTerm : public IloXmlInfo {
	
public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_NumLinTerm::getTagName(); }
  IloXmlInfo_NumLinTerm(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloNumLinTermI* numExprI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloNumLinTermI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);

protected:
  IloNumExpr getNumExprNonLinear(IloXmlReader reader, IloXmlElement* element);
  void getExprConstant(IloXmlReader reader, IloXmlElement* element, IloNum& constant);
  void getExprLinear(IloXmlReader reader, IloXmlElement* element, IloNumArray&Coef, IloNumVarArray& varArray);

  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloRtti* exprI){
    return getTagElement(writer, (IloNumLinTermI*)exprI);
  }
  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloNumLinTermI* exprI);

};

// ----------------------------------------------------------*- C++ -*-------
//    INCLUDE FOR IloXmlNumArrayElement
// ----------------------------------------------------------*- C++ -*-------

class IloNumArrayElementI;

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of IloXmlInfo_NumArrayElement 
 * makes it possible to serialize an IloNumArrayElement in XML. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_NumArrayElement : public IloXmlInfo {
	
public:
  static const char* getTagName();
  virtual const char* getTag() {
    return IloXmlInfo_NumArrayElement::getTagName();
  }
  IloXmlInfo_NumArrayElement(IloXmlContextI* context);

  virtual IloBool write(IloXmlWriter writer,
			IloNumArrayElementI* exprI, 
			IloXmlElement* masterElement);
  
  virtual IloBool write(IloXmlWriter writer, 
			const IloRtti* extractableI, 
			IloXmlElement* masterElement){
    return write(writer, (IloNumArrayElementI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

#endif
