// --------------------------------------------------------------------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef _ILOCPLEX_H
#define _ILOCPLEX_H

//
// To be included by applications using IloCplex.  It imports all include files
// needed for an IloCplex concert technology application.  Also inline
// implementation for various IloCplex member functions are provided here
//

#include <ilconcert/ilomodel.h>
#include <ilconcert/ilosys.h>
#include <ilcplex/ilocplexi.h>
#include <ilconcert/iloalg.h>
// #include <ilcplex/ilolpextr.h>
#include <ilcplex/ilm.h>
#include <ilcplex/ilogoals.h>


#if defined(CPXWIN32) || defined(CPXWIN64)
#define ILOCPLEX_VERSIONSTRING "@(#)ILOG Software: product CPLEX, library ilocplex.lib, version 10.2.0, date 03/30/2007 (C) 1997-2006 by ILOG"
#else
#define ILOCPLEX_VERSIONSTRING "@(#)ILOG Software: product CPLEX, library libilocplex.a, version 10.2.0, date 03/30/2007 (C) 1997-2006 by ILOG"
#endif


inline void
IloCplex::use(IloLPExtractor* trans) {
  IloAssert(getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert(trans, "accessing IloLPExtractor through 0 handle");
  getImpl()->use(trans);
}

inline void
IloCplex::getIntSetValue (const IloIntSetVar var, IloIntSet set) const {
  IloAssert(getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloIntSetVar through 0 handle");
  IloAssert (set.getImpl() != 0, "accessing IloIntSet through 0 handle");
  getImpl()->getIntSetValue(var.getImpl(), set);
}

inline IloNum
IloCplex::getDual  (const IloRange con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloRange through 0 handle");
 return (getImpl()->getDual(con.getImpl()));
}

#ifdef ILO_ADVANCED_CONCERT
inline IloNum
IloCplex::getAX  (const IloForAllRange con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloForAllRange through 0 handle");
 return (getImpl()->getAX(con.getImpl()));
}
inline IloNum
IloCplex::getDual  (const IloForAllRange con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloForAllRange through 0 handle");
 return (getImpl()->getDual(con.getImpl()));
}
inline IloNum
IloCplex::getSlack (const IloForAllRange con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloForAllRange through 0 handle");
 return (getImpl()->getSlack(con.getImpl()));
}
inline void
IloCplex::getForAllRanges (IloForAllRangeArray rows,
                                IloConstraint       forall) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (forall.getImpl() != 0, "accessing IloForAll through 0 handle");
  IloAssert (rows.getImpl() != 0, "array not initialized");
  getImpl()->getForAllRanges(rows, forall.getImpl());
}
inline IloNum
IloCplex::getLB(const IloForAllRange con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloForAllRange through 0 handle");
  return (getImpl()->getLB(con.getImpl()));
}
inline IloNum
IloCplex::getUB(const IloForAllRange con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloForAllRange through 0 handle");
  return (getImpl()->getUB(con.getImpl()));
}                             
inline IloBool
IloCplex::feasOpt(const IloRangeArray rngs,       const IloNumArray rnglb,
                                                  const IloNumArray rngub,
                  const IloForAllRangeArray frng, const IloNumArray frnglb,
                                                  const IloNumArray frngub,
                  const IloNumVarArray vars,      const IloNumArray varlb,
                                                  const IloNumArray varub) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  return getImpl()->feasOpt(rngs, rnglb, rngub,
                            frng, frnglb, frngub,
                            vars, varlb, varub);
                  }
#endif

inline IloNum
IloCplex::getSlack (const IloRange con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloRange through 0 handle");
  return (getImpl()->getSlack(con.getImpl()));
}

inline IloNum
IloCplex::getInfeasibility (const IloConstraint con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloConstraint through 0 handle");
  return (getImpl()->getInfeasibility(con.getImpl(), NULL));   
}

inline IloNum
IloCplex::getReducedCost(const IloNumVar con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloNumVar through 0 handle");
  return (getImpl()->getReducedCost(con.getImpl()));
}

inline IloNum
IloCplex::getReducedCost(const IloIntVar con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloIntVar through 0 handle");
  return (getImpl()->getReducedCost(con.getImpl()));
}

inline IloNum
IloCplex::getInfeasibility (const IloNumVar var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloNumVar through 0 handle");
  return (getImpl()->getInfeasibility(var.getImpl()));
}

inline IloNum
IloCplex::getInfeasibility (const IloIntVar var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloIntVar through 0 handle");
  return (getImpl()->getInfeasibility(var.getImpl()));
}

inline IloCplex::BasisStatus
IloCplex::getStatus(const IloNumVar con) const {
  return (getBasisStatus(con));
}

inline IloCplex::BasisStatus
IloCplex::getStatus(const IloRange con) const {
  return (getBasisStatus(con));
}

inline IloCplex::BasisStatus
IloCplex::getBasisStatus(const IloNumVar con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloNumVar through 0 handle");
  return (getImpl()->getBasisStatus(con.getImpl()));
}

inline
IloCplex::BasisStatus IloCplex::getBasisStatus(const IloIntVar con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloIntVar through 0 handle");
  return (getImpl()->getBasisStatus(con.getImpl()));
}

inline IloCplex::BasisStatus
IloCplex::getBasisStatus(const IloConstraint con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloConstraint through 0 handle");
  return (getImpl()->getBasisStatus(con.getImpl()));
}

inline void
IloCplex::getDuals  (IloNumArray val, const IloRangeArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (val.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getDuals(val, con);
}

#ifdef ILO_ADVANCED_CONCERT
inline void
IloCplex::getAX  (IloNumArray val, const IloForAllRangeArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (val.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getAX(val, con);
}
inline void
IloCplex::getDuals  (IloNumArray val, const IloForAllRangeArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (val.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getDuals(val, con);
}
inline void
IloCplex::getSlacks (IloNumArray val, const IloForAllRangeArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (val.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getSlacks(val, con);
}
#endif

inline void
IloCplex::getSlacks (IloNumArray val, const IloRangeArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (val.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getSlacks(val, con);
}

inline IloNum  
IloCplex::getInfeasibilities (IloNumArray infeas, const IloConstraintArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "array not initialized");
  IloAssert (infeas.getImpl() != 0, "array not initialized");
  return (getImpl()->getInfeasibilities(infeas, con));
}

inline void
IloCplex::getReducedCosts(IloNumArray val, const IloNumVarArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (val.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getReducedCosts(val, con);
}

inline void
IloCplex::getReducedCosts(IloNumArray val, const IloIntVarArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (val.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getReducedCosts(val, *(IloNumVarArray*)(void*)&con);
}

inline IloNum
IloCplex::getInfeasibilities (IloNumArray infeas, const IloNumVarArray var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (infeas.getImpl() != 0, "array not initialized");
  IloAssert (var.getImpl() != 0, "array not initialized");
  return (getImpl()->getInfeasibilities(infeas, var));
}

inline IloNum
IloCplex::getInfeasibilities (IloNumArray infeas, const IloIntVarArray var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (infeas.getImpl() != 0, "array not initialized");
  IloAssert (var.getImpl() != 0, "array not initialized");
  return (getImpl()->getInfeasibilities(infeas, *(IloNumVarArray*)(void*)&var));
}

inline void
IloCplex::getStatuses(IloCplex::BasisStatusArray stat,
                           const IloNumVarArray       con) const {
  getBasisStatuses(stat, con);
}

inline void
IloCplex::getStatuses(IloCplex::BasisStatusArray stat,
                           const IloRangeArray con) const {
  getBasisStatuses(stat, con);
}

inline void
IloCplex::getStatuses(IloCplex::BasisStatusArray cstat,
                           const IloNumVarArray       var,
                           IloCplex::BasisStatusArray rstat,
                           const IloRangeArray        con) const {
  getBasisStatuses(cstat, var, rstat, con);
}

inline void
IloCplex::setStatuses(const IloCplex::BasisStatusArray cstat,
                           const IloNumVarArray             var,
                           const IloCplex::BasisStatusArray rstat,
                           const IloRangeArray              con) {
  setBasisStatuses(cstat, var, rstat, con);
}

inline void
IloCplex::getBasisStatuses(IloCplex::BasisStatusArray stat,
                                const IloNumVarArray       con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (stat.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getBasisStatuses(stat, con);
}

inline void
IloCplex::getBasisStatuses(IloCplex::BasisStatusArray stat,
                                const IloIntVarArray       con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (stat.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getBasisStatuses(stat, *(IloNumVarArray*)(void*)&con);
}

inline void
IloCplex::getBasisStatuses(IloCplex::BasisStatusArray stat,
                                const IloConstraintArray   con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (stat.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getBasisStatuses(stat, con);
}

inline void
IloCplex::getBasisStatuses(IloCplex::BasisStatusArray cstat,
                                const IloNumVarArray       var,
                                IloCplex::BasisStatusArray rstat,
                                const IloConstraintArray   con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (cstat.getImpl() != 0, "array not initialized");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (rstat.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getBasisStatuses(cstat, var, rstat, con);
}

inline void
IloCplex::getBasisStatuses(IloCplex::BasisStatusArray cstat,
                                const IloIntVarArray       var,
                                IloCplex::BasisStatusArray rstat,
                                const IloConstraintArray   con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (cstat.getImpl() != 0, "array not initialized");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (rstat.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getBasisStatuses(cstat, *(IloNumVarArray*)(void*)&var, rstat, con);
}

inline void
IloCplex::setBasisStatuses(const IloCplex::BasisStatusArray cstat,
                                const IloNumVarArray             var,
                                const IloCplex::BasisStatusArray rstat,
                                const IloConstraintArray         con) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl()   == 0            ||
             cstat.getImpl() != 0            ||
             cstat.getSize() == var.getSize()  ,
             "arrays sizes don't match");
  IloAssert (rstat.getImpl() == 0            ||
             con.getImpl()   != 0            ||
             rstat.getSize() == con.getSize()  ,
             "arrays sizes don't match");
  getImpl()->setBasisStatuses(cstat, var, rstat, con);
}

inline void
IloCplex::setBasisStatuses(const IloCplex::BasisStatusArray cstat,
                                const IloIntVarArray             var,
                                const IloCplex::BasisStatusArray rstat,
                                const IloConstraintArray         con) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl()   == 0            ||
             cstat.getImpl() != 0            ||
             cstat.getSize() == var.getSize()  ,
             "arrays sizes don't match");
  IloAssert (rstat.getImpl() == 0            ||
             con.getImpl()   != 0            ||
             rstat.getSize() == con.getSize()  ,
             "arrays sizes don't match");
  getImpl()->setBasisStatuses(cstat, *(IloNumVarArray*)(void*)&var, rstat, con);
}

inline void
IloCplex::setVectors(const IloNumArray    x,
                          const IloNumArray    dj,
                          const IloNumVarArray var,
                          const IloNumArray    slack,
                          const IloNumArray    pi,
                          const IloRangeArray  rng) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() == 0              ||
             (x.getImpl() != 0            &&
              x.getSize() == var.getSize()  )  ,
             "arrays sizes don't match");
  IloAssert (var.getImpl() == 0            ||
             dj.getImpl()  == 0            ||
             dj.getSize()  == var.getSize()  ,
             "arrays sizes don't match");
  IloAssert (rng.getImpl() == 0               ||
             (pi.getImpl() != 0            &&
              pi.getSize() == rng.getSize()  )  ,
             "arrays sizes don't match");
  IloAssert (rng.getImpl()   == 0            ||
             slack.getImpl() == 0            ||
             slack.getSize() == rng.getSize()  ,
             "arrays sizes don't match");
  getImpl()->setVectors(x, dj, var, slack, pi, rng);
}

inline void
IloCplex::setVectors(const IloNumArray    x,
                          const IloNumArray    dj,
                          const IloIntVarArray var,
                          const IloNumArray    slack,
                          const IloNumArray    pi,
                          const IloRangeArray  rng) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() == 0              ||
             (x.getImpl() != 0            &&
              x.getSize() == var.getSize()  )  ,
             "arrays sizes don't match");
  IloAssert (var.getImpl() == 0            ||
             dj.getImpl()  == 0            ||
             dj.getSize()  == var.getSize()  ,
             "arrays sizes don't match");
  IloAssert (rng.getImpl() == 0               ||
             (pi.getImpl() != 0            &&
              pi.getSize() == rng.getSize()  )  ,
             "arrays sizes don't match");
  IloAssert (rng.getImpl()   == 0            ||
             slack.getImpl() == 0            ||
             slack.getSize() == rng.getSize()  ,
             "arrays sizes don't match");
  getImpl()->setVectors(x, dj, *(IloNumVarArray*)(void*)&var, slack, pi, rng);
}


inline IloNum
IloCplex::getAX(const IloRange con) const {
  IloAssert (getImpl() != 0,     "accessing IloCplex through 0 handle");
  IloAssert (con.getImpl() != 0, "accessing IloRange through 0 handle");
  return (getImpl()->getAX(con.getImpl()));
}

inline void
IloCplex::getAX(IloNumArray val, const IloRangeArray con) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (val.getImpl() != 0, "array not initialized");
  IloAssert (con.getImpl() != 0, "array not initialized");
  getImpl()->getAX(val, con);
}

inline void
IloCplex::getBoundSA(IloNumArray lblower, IloNumArray lbupper,
                          IloNumArray ublower, IloNumArray ubupper,
                          const IloNumVarArray cols) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (cols.getImpl() != 0, "array not initialized");
  getImpl()->getBoundSA(lblower, lbupper, ublower, ubupper, cols);
}

inline void
IloCplex::getBoundSA(IloNumArray lblower, IloNumArray lbupper,
                          IloNumArray ublower, IloNumArray ubupper,
                          const IloIntVarArray cols) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (cols.getImpl() != 0, "array not initialized");
  getImpl()->getBoundSA(lblower, lbupper, ublower, ubupper,
                        *(IloNumVarArray*)(void*)&cols);
}

inline void
IloCplex::getRangeSA(IloNumArray lblower, IloNumArray lbupper,
                          IloNumArray ublower, IloNumArray ubupper,
                          const IloRangeArray rows) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (rows.getImpl() != 0, "array not initialized");
  getImpl()->getRangeSA(lblower, lbupper, ublower, ubupper, rows);
}

inline void
IloCplex::getRHSSA(IloNumArray lower, IloNumArray upper,
                        const IloRangeArray rows) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (rows.getImpl() != 0, "array not initialized");
  getImpl()->getRHSSA(lower, upper, rows);
}

inline void
IloCplex::getObjSA(IloNumArray lower, IloNumArray upper,
                        const IloNumVarArray cols) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (cols.getImpl() != 0, "array not initialized");
  getImpl()->getObjSA(lower, upper, cols);
}

inline void
IloCplex::getObjSA(IloNumArray lower, IloNumArray upper,
                        const IloIntVarArray cols) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (cols.getImpl() != 0, "array not initialized");
  getImpl()->getObjSA(lower, upper, *(IloNumVarArray*)(void*)&cols);
}

inline void
IloCplex::getIIS(IloCplex::IISStatusArray cstat, IloNumVarArray var,
                      IloCplex::IISStatusArray rstat, IloConstraintArray rng)
                                                                        const {
  IloAssert (getImpl() != 0,     "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (rng.getImpl() != 0, "array not initialized");
  IloAssert (cstat.getImpl() != 0, "array not initialized");
  IloAssert (rstat.getImpl() != 0, "array not initialized");
  getImpl()->getIIS(cstat, var, rstat, rng);
}


inline IloBool
IloCplex::refineConflict(IloConstraintArray cons, IloNumArray prios) const {
  IloAssert (getImpl() != 0,     "accessing IloCplex through 0 handle");
  IloAssert (cons.getImpl() != 0, "array not initialized");
  IloAssert (prios.getImpl() != 0, "array not initialized");
  IloAssert (cons.getSize() == prios.getSize(), "array sizes don't match");
  return getImpl()->refineConflict(cons, prios);
}

/*
inline void
IloCplex::getConflict (IloCplex::ConflictStatusArray cstat,
                       IloNumVarArray                var,
                       IloCplex::ConflictStatusArray rstat,
                       IloConstraintArray            rng) const
{
  IloAssert (getImpl() != 0,     "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (rng.getImpl() != 0, "array not initialized");
  IloAssert (cstat.getImpl() != 0, "array not initialized");
  IloAssert (rstat.getImpl() != 0, "array not initialized");
  getImpl()->getConflict(cstat, var, rstat, rng);
}
*/


inline IloCplex::ConflictStatus
IloCplex::getConflict (IloConstraint rng) const
{
  IloAssert (getImpl() != 0,     "accessing IloCplex through 0 handle");
  IloAssert (rng.getImpl() != 0, "array not initialized");
  return getImpl()->getConflict(rng);
}

inline IloCplex::ConflictStatusArray
IloCplex::getConflict (IloConstraintArray rng) const
{
  IloAssert (getImpl() != 0,     "accessing IloCplex through 0 handle");
  IloAssert (rng.getImpl() != 0, "array not initialized");
  return getImpl()->getConflict(rng);
}


inline void
IloCplex::getDnorms(IloNumArray dnorms, const IloNumVarArray vars,
                                             const IloRangeArray  cons) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (dnorms.getImpl() != 0, "array not initialized");
  IloAssert (vars.getImpl() ||  cons.getImpl() != 0, "array not initialized");
  getImpl()->getDnorms(dnorms, vars, cons);
}

inline void
IloCplex::getDnorms(IloNumArray dnorms, const IloIntVarArray vars,
                                             const IloRangeArray  cons) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (dnorms.getImpl() != 0, "array not initialized");
  IloAssert (vars.getImpl() ||  cons.getImpl() != 0, "array not initialized");
  getImpl()->getDnorms(dnorms, *(IloNumVarArray*)(void*)&vars, cons);
}

inline void
IloCplex::getPnorms(IloNumArray cnorms, const IloNumVarArray vars,
                         IloNumArray rnorms, const IloRangeArray  cons) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (vars.getImpl() == 0  ||  cnorms.getImpl() != 0, "array not initialized");
  IloAssert (cons.getImpl() == 0  ||  rnorms.getImpl() != 0, "array not initialized");
  getImpl()->getPnorms(cnorms, vars, rnorms, cons);
}

inline void
IloCplex::getPnorms(IloNumArray cnorms, const IloIntVarArray vars,
                         IloNumArray rnorms, const IloRangeArray  cons) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (vars.getImpl() == 0  ||  cnorms.getImpl() != 0, "array not initialized");
  IloAssert (cons.getImpl() == 0  ||  rnorms.getImpl() != 0, "array not initialized");
  getImpl()->getPnorms(cnorms, *(IloNumVarArray*)(void*)&vars, rnorms, cons);
}

inline void
IloCplex::setDnorms(const IloNumArray norms, const IloNumVarArray vars,
                                                  const IloRangeArray  cons) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (norms.getImpl() != 0, "array not initialized");
  IloAssert (vars.getImpl() != 0, "array not initialized");
  IloAssert (cons.getImpl() != 0, "array not initialized");
  getImpl()->setDnorms(norms, vars, cons);
}

inline void
IloCplex::setDnorms(const IloNumArray norms, const IloIntVarArray vars,
                                                  const IloRangeArray  cons) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (norms.getImpl() != 0, "array not initialized");
  IloAssert (vars.getImpl() != 0, "array not initialized");
  IloAssert (cons.getImpl() != 0, "array not initialized");
  getImpl()->setDnorms(norms, *(IloNumVarArray*)(void*)&vars, cons);
}

inline void
IloCplex::setPnorms(const IloNumArray cnorms, const IloNumVarArray vars,
                         const IloNumArray rnorms, const IloRangeArray  cons) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (cnorms.getImpl() != 0, "array not initialized");
  IloAssert (rnorms.getImpl() != 0, "array not initialized");
  IloAssert (vars.getImpl() != 0, "array not initialized");
  IloAssert (cons.getImpl() != 0, "array not initialized");
  getImpl()->setPnorms(cnorms, vars, rnorms, cons);
}

inline void
IloCplex::setPnorms(const IloNumArray cnorms, const IloIntVarArray vars,
                         const IloNumArray rnorms, const IloRangeArray  cons) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (cnorms.getImpl() != 0, "array not initialized");
  IloAssert (rnorms.getImpl() != 0, "array not initialized");
  IloAssert (vars.getImpl() != 0, "array not initialized");
  IloAssert (cons.getImpl() != 0, "array not initialized");
  getImpl()->setPnorms(cnorms, *(IloNumVarArray*)(void*)&vars, rnorms, cons);
}

inline IloNum
IloCplex::getQuality(Quality q, IloNumVar*     var,
                                       IloConstraint* rng) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  return (getImpl()->getQuality(q, var, rng));
}

inline IloNum
IloCplex::getQuality(Quality q, IloConstraint* rng,
                                       IloNumVar*     var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  return (getImpl()->getQuality(q, var, rng));
}

inline void
IloCplex::setPriority  (IloNumVar var, IloNum pri) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->setPriority(var.getImpl(), pri);
}

inline void
IloCplex::setPriority  (IloIntVar var, IloNum pri) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->setPriority(var.getImpl(), pri);
}

inline void
IloCplex::setPriorities(const IloNumVarArray var, const IloNumArray pri) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (pri.getImpl() != 0, "array not initialized");
  getImpl()->setPriorities(var, pri);
}

inline void
IloCplex::setPriorities(const IloIntVarArray var, const IloNumArray pri) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (pri.getImpl() != 0, "array not initialized");
  getImpl()->setPriorities(*(IloNumVarArray*)(void*)&var, pri);
}

inline void
IloCplex::setDirection (IloNumVar var, BranchDirection dir) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->setDirection(var.getImpl(), dir);
}

inline void
IloCplex::setDirection (IloIntVar var, BranchDirection dir) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->setDirection(var.getImpl(), dir);
}

inline void
IloCplex::setDirections(const IloNumVarArray var, const BranchDirectionArray dir) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (dir.getImpl() != 0, "array not initialized");
  getImpl()->setDirections(var, dir);
}

inline void
IloCplex::setDirections(const IloIntVarArray var, const BranchDirectionArray dir) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (dir.getImpl() != 0, "array not initialized");
  getImpl()->setDirections(*(IloNumVarArray*)(void*)&var, dir);
}

inline void
IloCplex::delPriority  (IloNumVar var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->delPriority(var.getImpl());
}

inline void
IloCplex::delPriority  (IloIntVar var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->delPriority(var.getImpl());
}

inline void
IloCplex::delPriorities(const IloNumVarArray var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  getImpl()->delPriorities(var);
}

inline void
IloCplex::delPriorities(const IloIntVarArray var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  getImpl()->delPriorities(*(IloNumVarArray*)(void*)&var);
}

inline void
IloCplex::delDirection (IloNumVar var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->delDirection(var.getImpl());
}

inline void
IloCplex::delDirection (IloIntVar var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->delDirection(var.getImpl());
}

inline void
IloCplex::delDirections(const IloNumVarArray var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  getImpl()->delDirections(var);
}

inline void
IloCplex::delDirections(const IloIntVarArray var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  getImpl()->delDirections(*(IloNumVarArray*)(void*)&var);
}


inline IloNum IloCplex::getPriority(IloNumVar var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloNumVar through 0 handle");
  return (getImpl()->getPriority(var));
}

inline IloNum IloCplex::getPriority(IloIntVar var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloIntVar through 0 handle");
  return (getImpl()->getPriority(var));
}

inline IloCplex::BranchDirection IloCplex::getDirection(IloNumVar var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloNumVar through 0 handle");
  return (getImpl()->getDirection(var));
}

inline IloCplex::BranchDirection IloCplex::getDirection(IloIntVar var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloIntVar through 0 handle");
  return (getImpl()->getDirection(var));
}

inline void IloCplex::getPriorities(IloNumArray          pri,
                                    const IloNumVarArray var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloNumVarArray through 0 handle");
  IloAssert (pri.getImpl() != 0, "accessing IloVarArray through 0 handle");
  getImpl()->getPriorities(var, pri);
}

inline void IloCplex::getPriorities(IloNumArray          pri,
                                    const IloIntVarArray var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloIntVarArray through 0 handle");
  IloAssert (pri.getImpl() != 0, "accessing IloVarArray through 0 handle");
  getImpl()->getPriorities(*(IloNumVarArray*)(void*)&var, pri);
}

inline void IloCplex::getDirections(BranchDirectionArray dir,
                                    const IloNumVarArray var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloNumVarArray through 0 handle");
  IloAssert (dir.getImpl() != 0, "accessing IloVarArray through 0 handle");
  getImpl()->getDirections(var, dir);
}

inline void IloCplex::getDirections(BranchDirectionArray dir,
                                    const IloIntVarArray var) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "accessing IloIntVarArray through 0 handle");
  IloAssert (dir.getImpl() != 0, "accessing IloVarArray through 0 handle");
  getImpl()->getDirections(*(IloNumVarArray*)(void*)&var, dir);
}


inline void IloCplex::basicPresolve(const IloNumVarArray vars,
                                    IloNumArray          redlb,
                                    IloNumArray          redub,
                                    const IloRangeArray  rngs,
                                    IloBoolArray         redundant) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->basicPresolve(vars, redlb, redub, rngs, redundant);
}

inline void IloCplex::basicPresolve(const IloIntVarArray vars,
                                    IloNumArray          redlb,
                                    IloNumArray          redub,
                                    const IloRangeArray  rngs,
                                    IloBoolArray         redundant) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->basicPresolve(*(IloNumVarArray*)(void*)&vars, redlb, redub, rngs, redundant);
}

inline void IloCplex::presolve(IloCplex::Algorithm alg) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->presolve(alg);
}

inline void IloCplex::freePresolve() {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->freePresolve();
}

inline void IloCplex::getRay(IloNumArray vals, IloNumVarArray vars) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (vals.getImpl() != 0, "array not initialized");
  IloAssert (vars.getImpl() != 0, "array not initialized");
  getImpl()->getRay(vals, vars);
}

inline void
IloCplex::getDriebeekPenalties(IloNumArray          downratio,
                                    IloNumArray          upratio,
                                    const IloNumVarArray ind) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (ind.getImpl() != 0, "array not initialized");
  getImpl()->getDriebeekPenalties(ind, downratio, upratio);
}

inline void
IloCplex::getDriebeekPenalties(IloNumArray          downratio,
                                    IloNumArray          upratio,
                                    const IloIntVarArray ind) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (ind.getImpl() != 0, "array not initialized");
  getImpl()->getDriebeekPenalties(ind, downratio, upratio);
}

inline void
IloCplex::getDriebeekPenalties(IloNumArray         downratio,
                                    IloNumArray         upratio,
                                    const IloRangeArray ind) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (ind.getImpl() != 0, "array not initialized");
  getImpl()->getDriebeekPenalties(*(IloNumVarArray*)(void*)&ind,
                                  downratio, upratio);
}

inline void
IloCplex::getStrongBranch(IloNumArray          downratio,
                               IloNumArray          upratio,
                               const IloNumVarArray ind,
                               IloInt               itmax) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (ind.getImpl() != 0, "array not initialized");
  getImpl()->getStrongBranch(ind, downratio, upratio, itmax);
}

inline void
IloCplex::getStrongBranch(IloNumArray          downratio,
                               IloNumArray          upratio,
                               const IloIntVarArray ind,
                               IloInt               itmax) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (ind.getImpl() != 0, "array not initialized");
  getImpl()->getStrongBranch(ind, downratio, upratio, itmax);
}

inline void
IloCplex::getStrongBranch(IloNumArray         downratio,
                               IloNumArray         upratio,
                               const IloRangeArray ind,
                               IloInt              itmax) const {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (ind.getImpl() != 0, "array not initialized");
  getImpl()->getStrongBranch(*(IloNumVarArray*)(void*)&ind,
                             downratio, upratio, itmax);
}

inline void
IloCplex::protectVariables(const IloNumVarArray var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->protectVariables(var);
}

inline void
IloCplex::protectVariables(const IloIntVarArray var) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  getImpl()->protectVariables(*(IloNumVarArray*)(void*)&var);
}

inline IloNum
IloCplex::dualFarkas(IloConstraintArray  rng, IloNumArray y) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  return (getImpl()->dualFarkas(rng, y));
}

inline void
IloCplex::qpIndefCertificate(IloNumVarArray var, IloNumArray x) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (x.getImpl()   != 0, "array not initialized");
  getImpl()->qpIndefCertificate(var, x);
}

inline void
IloCplex::qpIndefCertificate(IloIntVarArray var, IloNumArray x) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  IloAssert (var.getImpl() != 0, "array not initialized");
  IloAssert (x.getImpl()   != 0, "array not initialized");
  getImpl()->qpIndefCertificate(*(IloNumVarArray*)(void*)&var, x);
}

inline IloBool IloCplex::feasOpt(const IloRangeArray rngs,
                                 const IloNumArray rnglb,
                                 const IloNumArray rngub,
                                 const IloNumVarArray vars,
                                 const IloNumArray varlb,
                                 const IloNumArray varub) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  return getImpl()->feasOpt(rngs, rnglb, rngub,
                            vars, varlb, varub);
}

inline IloBool IloCplex::feasOpt(const IloRangeArray rngs,
                                 const IloNumArray rnglb,
                                 const IloNumArray rngub,
                                 const IloIntVarArray vars,
                                 const IloNumArray varlb,
                                 const IloNumArray varub) {
  IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
  return getImpl()->feasOpt(rngs, rnglb, rngub,
                            *(IloNumVarArray*)(void*)&vars, varlb, varub);
}

inline IloBool IloCplex::feasOpt(const IloNumVarArray vars, const IloNumArray varlb,
                                                            const IloNumArray varub)
{
  return feasOpt(0, 0, 0, vars, varlb, varub);
}

inline IloBool IloCplex::feasOpt(const IloIntVarArray vars, const IloNumArray varlb,
                                                            const IloNumArray varub)
{
  return feasOpt(0, 0, 0, *(IloNumVarArray*)(void*)&vars, varlb, varub);
}

inline IloBool IloCplex::feasOpt(const IloRangeArray rngs,  const IloNumArray rnglb,
                                                            const IloNumArray rngub)
{
   IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
   IloAssert (rngs.getImpl()  != 0, "accessing constraint array through 0 handle");
   IloAssert (rnglb.getImpl() != 0, "accessing value array through 0 handle");
   IloAssert (rngub.getImpl() != 0, "accessing value array through 0 handle");
   IloAssert (rnglb.getSize() == rngs.getSize(),
              "incompatible array sizes");
   IloAssert (rngub.getSize() == rngs.getSize(),
              "incompatible array sizes");
   return getImpl()->feasOpt(rngs, rnglb, rngub, 0, 0, 0);
}


inline IloBool IloCplex::feasOpt(const IloConstraintArray rngs,
                                 const IloNumArray        relax)
{
   IloAssert (getImpl() != 0, "accessing IloCplex through 0 handle");
   IloAssert (rngs.getImpl()  != 0, "accessing constraint array through 0 handle");
   IloAssert (relax.getImpl() != 0, "accessing value array through 0 handle");
   IloAssert (relax.getSize() == rngs.getSize(),
              "incompatible array sizes");
   return getImpl()->feasOpt(rngs, relax);
}


ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, IloCplex::Algorithm alg);
ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, IloCplex::BasisStatus st);
ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, IloCplex::BranchDirection dir);


// --------------------------------------------------------------------------
// 
// callback macros
// 

#define ILOCOMMONCALLBACKSTUFF(name) \
IloCplex::CallbackI* duplicateCallback() const {\
   return (new (getEnv()) name##I(*this));\
}

#define ILOPRESOLVECALLBACK0(name) \
class name##I : public IloCplex::PresolveCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::PresolveCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOPRESOLVECALLBACK1(name, type1, x1) \
class name##I : public IloCplex::PresolveCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::PresolveCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOPRESOLVECALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::PresolveCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::PresolveCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOPRESOLVECALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::PresolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::PresolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOPRESOLVECALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::PresolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::PresolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOPRESOLVECALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::PresolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::PresolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOPRESOLVECALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::PresolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::PresolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOPRESOLVECALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::PresolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::PresolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOCONTINUOUSCALLBACK0(name) \
class name##I : public IloCplex::ContinuousCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::ContinuousCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOCONTINUOUSCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::ContinuousCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::ContinuousCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOCONTINUOUSCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::ContinuousCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::ContinuousCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOCONTINUOUSCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::ContinuousCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::ContinuousCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOCONTINUOUSCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::ContinuousCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::ContinuousCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOCONTINUOUSCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::ContinuousCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::ContinuousCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOCONTINUOUSCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::ContinuousCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::ContinuousCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOCONTINUOUSCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::ContinuousCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::ContinuousCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOSIMPLEXCALLBACK0(name) \
class name##I : public IloCplex::PrimalSimplexCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::PrimalSimplexCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOSIMPLEXCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::PrimalSimplexCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::PrimalSimplexCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOSIMPLEXCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::PrimalSimplexCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::PrimalSimplexCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOSIMPLEXCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::PrimalSimplexCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::PrimalSimplexCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOSIMPLEXCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::PrimalSimplexCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::PrimalSimplexCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOSIMPLEXCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::PrimalSimplexCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::PrimalSimplexCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOSIMPLEXCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::PrimalSimplexCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::PrimalSimplexCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOSIMPLEXCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::PrimalSimplexCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::PrimalSimplexCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()



#define ILOLPCALLBACK0(name) \
        ILOCONTINUOUSCALLBACK0(name)
#define ILOLPCALLBACK1(name, type1, x1) \
        ILOCONTINUOUSCALLBACK1(name, type1, x1)
#define ILOLPCALLBACK2(name, type1, x1, type2, x2) \
        ILOCONTINUOUSCALLBACK2(name, type1, x1, type2, x2)
#define ILOLPCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
        ILOCONTINUOUSCALLBACK3(name, type1, x1, type2, x2, type3, x3)
#define ILOLPCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
        ILOCONTINUOUSCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4)
#define ILOLPCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                       type5, x5) \
        ILOCONTINUOUSCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5)
#define ILOLPCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                       type5, x5) \
        ILOCONTINUOUSCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5, type6, x6)
#define ILOLPCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                       type5, x5) \
        ILOCONTINUOUSCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5, type6, x6, type7, x7)

#define ILOPRIMALSIMPLEXCALLBACK0(name) \
        ILOSIMPLEXCALLBACK0(name)
#define ILOPRIMALSIMPLEXCALLBACK1(name, type1, x1) \
        ILOSIMPLEXCALLBACK1(name, type1, x1)
#define ILOPRIMALSIMPLEXCALLBACK2(name, type1, x1, type2, x2) \
        ILOSIMPLEXCALLBACK2(name, type1, x1, type2, x2)
#define ILOPRIMALSIMPLEXCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
        ILOSIMPLEXCALLBACK3(name, type1, x1, type2, x2, type3, x3)
#define ILOPRIMALSIMPLEXCALLBACK4(name, type1, x1, type2, x2, type3, x3, \
                                  type4, x4) \
        ILOSIMPLEXCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4)
#define ILOPRIMALSIMPLEXCALLBACK5(name, type1, x1, type2, x2, type3, x3, \
                                  type4, x4, type5, x5) \
        ILOSIMPLEXCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5)
#define ILOPRIMALSIMPLEXCALLBACK6(name, type1, x1, type2, x2, type3, x3, \
                                  type4, x4, type5, x5) \
        ILOSIMPLEXCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5, type6, x6)
#define ILOPRIMALSIMPLEXCALLBACK7(name, type1, x1, type2, x2, type3, x3, \
                                  type4, x4, type5, x5) \
        ILOSIMPLEXCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5, type6, x6, type7, x7)

#define ILODUALSIMPLEXCALLBACK0(name) \
        ILOSIMPLEXCALLBACK0(name)
#define ILODUALSIMPLEXCALLBACK1(name, type1, x1) \
        ILOSIMPLEXCALLBACK1(name, type1, x1)
#define ILODUALSIMPLEXCALLBACK2(name, type1, x1, type2, x2) \
        ILOSIMPLEXCALLBACK2(name, type1, x1, type2, x2)
#define ILODUALSIMPLEXCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
        ILOSIMPLEXCALLBACK3(name, type1, x1, type2, x2, type3, x3)
#define ILODUALSIMPLEXCALLBACK4(name, type1, x1, type2, x2, type3, x3, \
                                  type4, x4) \
        ILOSIMPLEXCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4)
#define ILODUALSIMPLEXCALLBACK5(name, type1, x1, type2, x2, type3, x3, \
                                  type4, x4, type5, x5) \
        ILOSIMPLEXCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5)
#define ILODUALSIMPLEXCALLBACK6(name, type1, x1, type2, x2, type3, x3, \
                                  type4, x4, type5, x5) \
        ILOSIMPLEXCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5, type6, x6)
#define ILODUALSIMPLEXCALLBACK7(name, type1, x1, type2, x2, type3, x3, \
                                  type4, x4, type5, x5) \
        ILOSIMPLEXCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, \
                               x4, type5, x5, type6, x6, type7, x7)


#define ILOBARRIERCALLBACK0(name) \
class name##I : public IloCplex::BarrierCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::BarrierCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOBARRIERCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::BarrierCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::BarrierCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOBARRIERCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::BarrierCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::BarrierCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOBARRIERCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::BarrierCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::BarrierCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOBARRIERCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::BarrierCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::BarrierCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOBARRIERCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::BarrierCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::BarrierCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOBARRIERCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::BarrierCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::BarrierCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOBARRIERCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::BarrierCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::BarrierCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILONETWORKCALLBACK0(name) \
class name##I : public IloCplex::NetworkCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::NetworkCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILONETWORKCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::NetworkCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::NetworkCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILONETWORKCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::NetworkCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::NetworkCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILONETWORKCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::NetworkCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::NetworkCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILONETWORKCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::NetworkCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::NetworkCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILONETWORKCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::NetworkCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::NetworkCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILONETWORKCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::NetworkCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::NetworkCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILONETWORKCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::NetworkCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::NetworkCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOMIPCALLBACK0(name) \
class name##I : public IloCplex::MIPCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::MIPCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOMIPCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::MIPCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::MIPCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOMIPCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::MIPCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::MIPCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOMIPCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::MIPCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::MIPCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOMIPCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::MIPCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::MIPCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOMIPCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::MIPCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::MIPCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOMIPCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::MIPCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::MIPCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOMIPCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::MIPCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::MIPCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOPROBINGCALLBACK0(name) \
class name##I : public IloCplex::ProbingCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::ProbingCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOPROBINGCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::ProbingCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::ProbingCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOPROBINGCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::ProbingCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::ProbingCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOPROBINGCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::ProbingCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::ProbingCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOPROBINGCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::ProbingCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::ProbingCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOPROBINGCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::ProbingCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::ProbingCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOPROBINGCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::ProbingCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::ProbingCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOPROBINGCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::ProbingCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::ProbingCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOFRACTIONALCUTCALLBACK0(name) \
class name##I : public IloCplex::FractionalCutCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::FractionalCutCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOFRACTIONALCUTCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::FractionalCutCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::FractionalCutCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOFRACTIONALCUTCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::FractionalCutCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::FractionalCutCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOFRACTIONALCUTCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::FractionalCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::FractionalCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOFRACTIONALCUTCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::FractionalCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::FractionalCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOFRACTIONALCUTCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::FractionalCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::FractionalCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOFRACTIONALCUTCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::FractionalCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::FractionalCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOFRACTIONALCUTCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::FractionalCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::FractionalCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOFLOWMIRCUTCALLBACK0(name) \
class name##I : public IloCplex::FlowMIRCutCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::FlowMIRCutCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOFLOWMIRCUTCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::FlowMIRCutCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::FlowMIRCutCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOFLOWMIRCUTCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::FlowMIRCutCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::FlowMIRCutCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOFLOWMIRCUTCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::FlowMIRCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::FlowMIRCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOFLOWMIRCUTCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::FlowMIRCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::FlowMIRCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOFLOWMIRCUTCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::FlowMIRCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::FlowMIRCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOFLOWMIRCUTCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::FlowMIRCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::FlowMIRCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOFLOWMIRCUTCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::FlowMIRCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::FlowMIRCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILODISJUNCTIVECUTCALLBACK0(name) \
class name##I : public IloCplex::DisjunctiveCutCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILODISJUNCTIVECUTCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::DisjunctiveCutCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::DisjunctiveCutCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILODISJUNCTIVECUTCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::DisjunctiveCutCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::DisjunctiveCutCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILODISJUNCTIVECUTCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::DisjunctiveCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::DisjunctiveCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILODISJUNCTIVECUTCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::DisjunctiveCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::DisjunctiveCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILODISJUNCTIVECUTCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::DisjunctiveCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::DisjunctiveCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILODISJUNCTIVECUTCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::DisjunctiveCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::DisjunctiveCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILODISJUNCTIVECUTCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::DisjunctiveCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::DisjunctiveCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOCROSSOVERCALLBACK0(name) \
class name##I : public IloCplex::CrossoverCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::CrossoverCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOCROSSOVERCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::CrossoverCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::CrossoverCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOCROSSOVERCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::CrossoverCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::CrossoverCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOCROSSOVERCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::CrossoverCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::CrossoverCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOCROSSOVERCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::CrossoverCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::CrossoverCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOCROSSOVERCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::CrossoverCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::CrossoverCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOCROSSOVERCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::CrossoverCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::CrossoverCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOCROSSOVERCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::CrossoverCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::CrossoverCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOUSERCUTCALLBACK0(name) \
class name##I : public IloCplex::UserCutCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::UserCutCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOUSERCUTCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::UserCutCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::UserCutCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOUSERCUTCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::UserCutCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::UserCutCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOUSERCUTCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::UserCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::UserCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOUSERCUTCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::UserCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::UserCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOUSERCUTCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::UserCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::UserCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOUSERCUTCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::UserCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::UserCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOUSERCUTCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::UserCutCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::UserCutCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOLAZYCONSTRAINTCALLBACK0(name) \
class name##I : public IloCplex::LazyConstraintCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::LazyConstraintCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOLAZYCONSTRAINTCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::LazyConstraintCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::LazyConstraintCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOLAZYCONSTRAINTCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::LazyConstraintCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::LazyConstraintCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOLAZYCONSTRAINTCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::LazyConstraintCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::LazyConstraintCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOLAZYCONSTRAINTCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::LazyConstraintCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::LazyConstraintCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOLAZYCONSTRAINTCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::LazyConstraintCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::LazyConstraintCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOLAZYCONSTRAINTCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::LazyConstraintCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::LazyConstraintCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOLAZYCONSTRAINTCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::LazyConstraintCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::LazyConstraintCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()

#define ILOCUTCALLBACK0(name) ILOLAZYCONSTRAINTCALLBACK0(name)
#define ILOCUTCALLBACK1(name, type1, x1) \
        ILOLAZYCONSTRAINTCALLBACK1(name, type1, x1)
#define ILOCUTCALLBACK2(name, type1, x1, type2, x2) \
        ILOLAZYCONSTRAINTCALLBACK2(name, type1, x1, type2, x2)
#define ILOCUTCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
        ILOLAZYCONSTRAINTCALLBACK3(name, type1, x1, type2, x2, type3, x3)
#define ILOCUTCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
        ILOLAZYCONSTRAINTCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4)
#define ILOCUTCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
        ILOLAZYCONSTRAINTCALLBACK5(name, type1, x1, type2, x2, type3, x3, \
                                         type4, x4, type5, x5)
#define ILOCUTCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                              type5, x5, type6, x6) \
        ILOLAZYCONSTRAINTCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                                         type5, x5, type6, x6)
#define ILOCUTCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                              type5, x5, type6, x6, type7, x7) \
        ILOLAZYCONSTRAINTCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                                         type5, x5, type6, x6, type7, x7)


#define ILONODECALLBACK0(name) \
class name##I : public IloCplex::NodeCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::NodeCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILONODECALLBACK1(name, type1, x1) \
class name##I : public IloCplex::NodeCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::NodeCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILONODECALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::NodeCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::NodeCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILONODECALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::NodeCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::NodeCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILONODECALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::NodeCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::NodeCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILONODECALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::NodeCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::NodeCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILONODECALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::NodeCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::NodeCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILONODECALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::NodeCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::NodeCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOINCUMBENTCALLBACK0(name) \
class name##I : public IloCplex::IncumbentCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::IncumbentCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOINCUMBENTCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::IncumbentCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::IncumbentCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOINCUMBENTCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::IncumbentCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::IncumbentCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOINCUMBENTCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::IncumbentCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::IncumbentCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOINCUMBENTCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::IncumbentCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::IncumbentCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOINCUMBENTCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::IncumbentCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::IncumbentCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOINCUMBENTCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::IncumbentCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::IncumbentCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOINCUMBENTCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::IncumbentCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::IncumbentCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOBRANCHCALLBACK0(name) \
class name##I : public IloCplex::BranchCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::BranchCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOBRANCHCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::BranchCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::BranchCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOBRANCHCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::BranchCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::BranchCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOBRANCHCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::BranchCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::BranchCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOBRANCHCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::BranchCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::BranchCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOBRANCHCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::BranchCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::BranchCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOBRANCHCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::BranchCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::BranchCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOBRANCHCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::BranchCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::BranchCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()


#define ILOHEURISTICCALLBACK0(name) \
class name##I : public IloCplex::HeuristicCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::HeuristicCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOHEURISTICCALLBACK1(name, type1, x1) \
class name##I : public IloCplex::HeuristicCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::HeuristicCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOHEURISTICCALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::HeuristicCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : \
  IloCplex::HeuristicCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOHEURISTICCALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::HeuristicCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::HeuristicCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOHEURISTICCALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::HeuristicCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::HeuristicCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOHEURISTICCALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::HeuristicCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::HeuristicCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOHEURISTICCALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::HeuristicCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::HeuristicCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOHEURISTICCALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::HeuristicCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::HeuristicCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()

#define ILOSOLVECALLBACK0(name) \
class name##I : public IloCplex::SolveCallbackI {\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env) : IloCplex::SolveCallbackI(env) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env) {\
  return (IloCplex::Callback(new (env) name##I(env)));\
}\
void name##I::main()

#define ILOSOLVECALLBACK1(name, type1, x1) \
class name##I : public IloCplex::SolveCallbackI {\
  type1 x1;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1) : IloCplex::SolveCallbackI(env), x1(xx1) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1) {\
  return (IloCplex::Callback(new (env) name##I(env, x1)));\
}\
void name##I::main()

#define ILOSOLVECALLBACK2(name, type1, x1, type2, x2) \
class name##I : public IloCplex::SolveCallbackI {\
  type1 x1;\
  type2 x2;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2) : IloCplex::SolveCallbackI(env), x1(xx1), x2(xx2) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2)));\
}\
void name##I::main()

#define ILOSOLVECALLBACK3(name, type1, x1, type2, x2, type3, x3) \
class name##I : public IloCplex::SolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3) : \
  IloCplex::SolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3)));\
}\
void name##I::main()

#define ILOSOLVECALLBACK4(name, type1, x1, type2, x2, type3, x3, type4, x4) \
class name##I : public IloCplex::SolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4) \
    : IloCplex::SolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4)));\
}\
void name##I::main()

#define ILOSOLVECALLBACK5(name, type1, x1, type2, x2, type3, x3, type4, x4, type5, x5) \
class name##I : public IloCplex::SolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5) \
    : IloCplex::SolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, type5 x5) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5)));\
}\
void name##I::main()

#define ILOSOLVECALLBACK6(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6) \
class name##I : public IloCplex::SolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6) \
    : IloCplex::SolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6)));\
}\
void name##I::main()

#define ILOSOLVECALLBACK7(name, type1, x1, type2, x2, type3, x3, type4, x4, \
                        type5, x5, type6, x6, type7, x7) \
class name##I : public IloCplex::SolveCallbackI {\
  type1 x1;\
  type2 x2;\
  type3 x3;\
  type4 x4;\
  type5 x5;\
  type6 x6;\
  type7 x7;\
public:\
  ILOCOMMONCALLBACKSTUFF(name) \
  name##I(IloEnv env, type1 xx1, type2 xx2, type3 xx3, type4 xx4, type5 xx5, type6 xx6, type7 xx7) \
    : IloCplex::SolveCallbackI(env), x1(xx1), x2(xx2), x3(xx3), x4(xx4), x5(xx5), x6(xx6), x7(xx7) {}\
  void main();\
};\
IloCplex::Callback name(IloEnv env, type1 x1, type2 x2, type3 x3, type4 x4, \
                        type5 x5, type6 x6, type7 x7) {\
  return (IloCplex::Callback(new (env) name##I(env, x1, x2, x3, x4, x5, x6, x7)));\
}\
void name##I::main()

#endif
